@@ -558,8 +558,12 @@ do_read_directory (GnomeVFSMethod *method,
 		result = gnome_vfs_directory_read_next (wrapped_handle->u.wrapped.handle,
 							file_info);
 	} else {
+		if (gnome_vfs_context_check_cancellation (context)) {
+			return GNOME_VFS_ERROR_INTERRUPTED;
+		}
+
 		result = medusa_search_service_connection_read_search_result 
-			(wrapped_handle->u.real.connection, &result_uri);
+			(wrapped_handle->u.real.connection, context, &result_uri);
 
 		if (result == GNOME_VFS_OK) {
 			/* FIXME bugzilla.eazel.com 2619:
