@@ -27,10 +27,6 @@
 
 #include <config.h>
 #include <glib.h>
-
-#include <medusa-file-index-encoders.h>
-#include <medusa-conf.h>
-#include <medusa-test.h>
 #include <time.h>
 #include <sys/types.h>
 #include <string.h>
@@ -38,16 +34,16 @@
 #include <sys/stat.h>
 #include <fcntl.h> 
 
+#include "medusa-file-index-encoders.h"
+#include "medusa-conf.h"
+#include "medusa-test.h"
 
 void
 medusa_file_database_uri_number_encode (char *result,
                                         int *uri_number,
                                         MedusaFileSystemDB *file_system_db)
 {
-        memcpy (result, uri_number, sizeof (int));
-        /* strncpy (result, medusa_int_to_bytes (owner, MEDUSA_FILE_INDEX_OWNER_FIELD_SIZE),
-           medusa_rdb_field_get_size (file_system_db->file_database->file->field_info, 
-           MEDUSA_FILE_INDEX_OWNER_FIELD_TITLE));*/
+        memcpy (result, uri_number, MEDUSA_FILE_INDEX_URI_NUMBER_FIELD_SIZE);
 }
                                         
 
@@ -56,17 +52,11 @@ medusa_file_database_mime_type_encode (char *result,
 				       char *mime_type,
 				       MedusaFileSystemDB *file_system_db)
 {
-        char *bytes; 
+        MedusaToken token;
 
-        bytes = medusa_token_to_bytes (medusa_string_to_token (file_system_db->mime_types,
-                                                               mime_type),
-                                       medusa_rdb_field_get_size (file_system_db->file_database->file->field_info, 
-                                                                  MEDUSA_FILE_INDEX_MIME_TYPE_FIELD_TITLE));
-        memcpy (result,
-                bytes,
-                medusa_rdb_field_get_size (file_system_db->file_database->file->field_info, 
-                                           MEDUSA_FILE_INDEX_MIME_TYPE_FIELD_TITLE));
-        g_free (bytes);
+        token = medusa_string_list_store_string (file_system_db->mime_types,
+                                                 mime_type);
+        medusa_convert_token_to_bytes (result, token);
 }
 
 void
@@ -74,19 +64,16 @@ medusa_file_database_mtime_encode (char *result,
 				   time_t *mtime,
                                    MedusaFileSystemDB *file_system_db)
 {
-        memcpy (result, (char *) mtime, sizeof (time_t));
+        memcpy (result, (char *) mtime, MEDUSA_FILE_INDEX_MTIME_FIELD_SIZE);
 }
 
 
 void
 medusa_file_database_owner_encode (char *result,
-				   int *owner,
+				   uid_t *owner,
                                    MedusaFileSystemDB *file_system_db)
 {
-        memcpy (result, owner, sizeof (int));
-        /* strncpy (result, medusa_int_to_bytes (owner, MEDUSA_FILE_INDEX_OWNER_FIELD_SIZE),
-           medusa_rdb_field_get_size (file_system_db->file_database->file->field_info, 
-           MEDUSA_FILE_INDEX_OWNER_FIELD_TITLE));*/
+        memcpy (result, owner, MEDUSA_FILE_INDEX_OWNER_FIELD_SIZE);
 }
 
 void
@@ -95,20 +82,16 @@ medusa_file_database_group_encode (char *result,
                                    MedusaFileSystemDB *file_system_db)
 	
 {
-        memcpy (result, group, 
-                medusa_rdb_field_get_size (file_system_db->file_database->file->field_info, 
-                                           MEDUSA_FILE_INDEX_GROUP_FIELD_TITLE));
+        memcpy (result, group, MEDUSA_FILE_INDEX_GROUP_FIELD_SIZE);
 }
 
 void
 medusa_file_database_permissions_encode (char *result,
-					 int *permissions,
+					 GnomeVFSFilePermissions *permissions,
                                          MedusaFileSystemDB *file_system_db)
      
 {
-        memcpy (result, permissions, 
-                medusa_rdb_field_get_size (file_system_db->file_database->file->field_info, 
-                                           MEDUSA_FILE_INDEX_PERMISSIONS_FIELD_TITLE));
+        memcpy (result, permissions, MEDUSA_FILE_INDEX_PERMISSIONS_FIELD_SIZE);
 }
 
 
@@ -118,17 +101,15 @@ medusa_file_database_size_encode (char *result,
                                   MedusaFileSystemDB *file_system_db)
      
 {
-        memcpy (result, size,
-                medusa_rdb_field_get_size (file_system_db->file_database->file->field_info, 
-                                           MEDUSA_FILE_INDEX_SIZE_FIELD_TITLE));
+        memcpy (result, size, MEDUSA_FILE_INDEX_SIZE_FIELD_SIZE);
 }
 
 void
-medusa_file_database_uri_number_decode (int *uri_number,
+medusa_file_database_uri_number_decode (gint32 *uri_number,
                                         char *field,
                                         gpointer data)
 {
-        memcpy (uri_number, field, sizeof (int));
+        memcpy (uri_number, field, MEDUSA_FILE_INDEX_URI_NUMBER_FIELD_SIZE);
 }
 
 
@@ -137,12 +118,15 @@ medusa_file_database_mime_type_decode (char *mime_type_result,
 				       char *field,
 				       MedusaFileSystemDB *file_system_db)
 {
+        MedusaToken token;
+
+        medusa_convert_bytes_to_token (&token,
+                                       field);
         strncpy (mime_type_result, 
-                 medusa_token_to_string (file_system_db->mime_types,
-                                         medusa_bytes_to_token (field,
-                                                                medusa_rdb_field_get_size (file_system_db->file_database->file->field_info, 
-                                                                                           MEDUSA_FILE_INDEX_MIME_TYPE_FIELD_TITLE))),
-                 NAME_MAX);
+                 medusa_string_list_get_string (file_system_db->mime_types,
+                                                token),
+                 NAME_MAX - 1);
+        mime_type_result[NAME_MAX - 1] = 0;
 }
 
 void
@@ -150,32 +134,32 @@ medusa_file_database_mtime_decode (time_t *mtime,
 				   char *field,
 				   gpointer data)
 {
-        memcpy (mtime, field, sizeof (time_t));
+        memcpy (mtime, field, MEDUSA_FILE_INDEX_MTIME_FIELD_SIZE);
 }
 
 
 void
-medusa_file_database_owner_decode (int *owner,
+medusa_file_database_owner_decode (uid_t *owner,
 				   char *field,
 				   gpointer data)
 {
-        memcpy (owner, field, sizeof (int));
+        memcpy (owner, field, MEDUSA_FILE_INDEX_OWNER_FIELD_SIZE);
 }
 
 void
-medusa_file_database_group_decode (unsigned long *group,
+medusa_file_database_group_decode (gid_t *group,
 				   char *field,
 				   gpointer data)
 {
-        *group = *(gid_t *) field;
+        memcpy (group, field, MEDUSA_FILE_INDEX_GROUP_FIELD_SIZE);
 }
 
 void
-medusa_file_database_permissions_decode (int *permissions,
+medusa_file_database_permissions_decode (GnomeVFSFilePermissions *permissions,
 					 char *field,
 					 gpointer data)
 {
-        *permissions = *(int *) field;
+        memcpy (permissions, field, MEDUSA_FILE_INDEX_PERMISSIONS_FIELD_SIZE);
 }
 
 void
@@ -183,7 +167,7 @@ medusa_file_database_size_decode (size_t *size,
                                   char *field,
                                   gpointer data)
 {
-        *size = *(size_t *) field;
+        memcpy (size, field, MEDUSA_FILE_INDEX_SIZE_FIELD_SIZE);
 }
 
 void
@@ -192,12 +176,10 @@ medusa_file_database_keywords_encode (char *result,
                                       MedusaFileSystemDB *file_system_db)
 {
         MedusaToken token;
-        char *bytes;
 
-        token = medusa_string_to_token (file_system_db->keyword_sets, keywords);
-        bytes = medusa_token_to_bytes (token, MEDUSA_FILE_INDEX_KEYWORDS_FIELD_SIZE);
-        memcpy (result, bytes, MEDUSA_FILE_INDEX_KEYWORDS_FIELD_SIZE);
-        g_free (bytes);
+        token = medusa_string_list_store_string (file_system_db->keyword_sets, keywords);
+        medusa_convert_token_to_bytes (result,
+                                       token);
 }
 
 void
@@ -206,25 +188,27 @@ medusa_file_database_keywords_decode (char *result,
                                       MedusaFileSystemDB *file_system_db)
 {
         MedusaToken token;
-        char *keywords;
+        const char *keywords;
 
-        token = medusa_bytes_to_token (field, MEDUSA_FILE_INDEX_KEYWORDS_FIELD_SIZE);
+        medusa_convert_bytes_to_token (&token, field);
         if (token == 0) {
                 result[0] = '\0';
                 return;
         }
-        keywords = medusa_token_to_string (file_system_db->keyword_sets, token);
+        keywords = medusa_string_list_get_string (file_system_db->keyword_sets, token);
         /* FIXME bugzilla.eazel.com 2646: 
            Truncating the list of keywords sucks, but it's the
            * best we can do given this encoding/decoding API.
            */
-        strncpy (result, keywords, NAME_MAX);
+        strncpy (result, keywords, NAME_MAX - 1);
+        result[NAME_MAX - 1] = 0;
+
 }
 
 void 
 medusa_file_index_encoders_test (void)
 {
-        
+#if 0        
         char buffer[NAME_MAX];
         char *name, name_back[NAME_MAX];
         time_t time_v, time_back;
@@ -349,4 +333,5 @@ medusa_file_index_encoders_test (void)
         medusa_file_database_size_encode (buffer, &size, file_system_db);
         medusa_file_database_size_decode (&size_back, buffer, file_system_db);
         MEDUSA_TEST_INTEGER_RESULT (size_back, size);
+#endif
 }
