@@ -28,14 +28,15 @@
 #ifndef MEDUSA_FILE_INDEX_ENCODERS_H
 #define MEDUSA_FILE_INDEX_ENCODERS_H
 
-#include <medusa-file-index.h>
+#include "medusa-file-index.h"
+#include <libgnomevfs/gnome-vfs-types.h>
 
 #ifndef NAME_MAX
 #define NAME_MAX 512
 #endif
 
 void medusa_file_database_uri_number_encode  (char               *result,
-                                              int                *uri_number,
+                                              gint32             *uri_number,
                                               MedusaFileSystemDB *file_system_db);
 void medusa_file_database_mime_type_encode   (char               *result,
                                               char               *mime_type,
@@ -44,13 +45,13 @@ void medusa_file_database_mtime_encode       (char               *result,
                                               time_t             *time,
                                               MedusaFileSystemDB *file_system_db);
 void medusa_file_database_owner_encode       (char               *result,
-                                              int                *owner,
+                                              uid_t              *owner,
                                               MedusaFileSystemDB *file_system_db);
 void medusa_file_database_group_encode       (char               *result,
                                               gid_t              *group,
                                               MedusaFileSystemDB *file_system_db);
 void medusa_file_database_permissions_encode (char               *result,
-                                              int                *permissions,
+                                              GnomeVFSFilePermissions *permissions,
                                               MedusaFileSystemDB *file_system_db);
 void medusa_file_database_size_encode        (char               *result,
                                               size_t             *size,
@@ -70,13 +71,13 @@ void medusa_file_database_keywords_encode    (char               *result,
 void medusa_file_database_mtime_decode       (time_t             *mtime,
                                               char               *field,
                                               gpointer            data);
-void medusa_file_database_owner_decode       (int                *owner,
+void medusa_file_database_owner_decode       (uid_t              *owner,
                                               char               *field,
                                               gpointer            data);
-void medusa_file_database_group_decode       (unsigned long      *group,
+void medusa_file_database_group_decode       (gid_t              *group,
                                               char               *field,
                                               gpointer            data);
-void medusa_file_database_permissions_decode (int                *permissions,
+void medusa_file_database_permissions_decode (GnomeVFSFilePermissions *permissions,
                                               char               *field,
                                               gpointer            data);
 void medusa_file_database_size_decode        (size_t             *size,
