@@ -31,23 +31,29 @@
 #include "medusa-file-index-encoders.h"
 #include "medusa-file-index-queries.h"
 #include "medusa-file-information.h"
+#include "medusa-index-filenames.h"
 #include "medusa-keyword-set.h"
+#include "medusa-query-clauses.h"
 #include "medusa-search-uri.h"
+#include "medusa-string-list.h"
 #include <libgnomevfs/gnome-vfs-utils.h>
-#include <libmedusa/medusa-utils.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
+#define FILE_ATTRIBUTE_DATABASE_PREFIX  "file-system-db"
+#define MIME_TYPE_PREFIX  "mime-types"
+#define KEYWORD_SETS_PREFIX  "keyword-sets"
 #define FILE_INDEX_METAINFO_LENGTH 100
 
 /* Check that the field information in the header (just the titles) matches what we
    expected. */
-static MedusaRDBFieldInfo *  file_index_field_info          (void);
-static MedusaQueryClauses *  file_index_clauses             (void);
-static void                  read_index_info_from_database  (MedusaFileSystemDB *file_system_db);
-static void                  write_index_info_to_database   (MedusaFileSystemDB *file_system_db,
-                                                             int last_indexing_timex);
+static MedusaRDBFieldInfo *  file_index_field_info                              (void);
+static MedusaQueryClauses *  file_index_clauses_using_index_only                (void);
+static MedusaQueryClauses *  file_index_clauses_using_index_and_uid             (void);
+static void                  read_index_info_from_database                      (MedusaFileSystemDB *file_system_db);
+static void                  write_index_info_to_database                       (MedusaFileSystemDB *file_system_db,
+                                                                                 int last_indexing_time);
 
 typedef void (* AddFieldCallback) (const char *title, int size,
                                    MedusaRDBEncodeFunc encoder,
@@ -115,22 +121,19 @@ medusa_rdb_file_add_field_cover (const char *title, int size,
 }
 
 MedusaFileSystemDB *
-medusa_file_system_db_new (const char *root_directory,
-                           const char *db_file_name,
-                           MedusaHash *file_names,
-                           MedusaHash *directory_names,
-                           const char *mime_type_hash,
-                           const char *keyword_sets_file_name)
+medusa_file_system_db_new (const char *index_name)
 {
         MedusaFileSystemDB *file_system_db;
         MedusaRDBFile *db_file;
-
+        char *file_system_db_file_name, *mime_type_list_name, *keyword_sets_list_name;
+        
         file_system_db = g_new0 (MedusaFileSystemDB, 1);
 
-        g_return_val_if_fail (access (db_file_name, X_OK) == -1, NULL);
-        g_return_val_if_fail (access (keyword_sets_file_name, X_OK) == -1, NULL);
-
-        db_file = medusa_rdb_file_new (db_file_name, FILE_INDEX_METAINFO_LENGTH);
+        file_system_db_file_name = medusa_generate_index_filename (FILE_ATTRIBUTE_DATABASE_PREFIX, 
+                                                                   index_name, 
+                                                                   TRUE);
+        db_file = medusa_rdb_file_new (file_system_db_file_name, FILE_INDEX_METAINFO_LENGTH);
+        g_free (file_system_db_file_name);
 
         /* Create fields and reserved first record */
         add_fields (medusa_rdb_file_add_field_cover, db_file);
@@ -141,16 +144,20 @@ medusa_file_system_db_new (const char *root_directory,
         file_system_db->version = g_strdup (MEDUSA_INDEX_FORMAT_VERSION);
 
         file_system_db->indexing_start_time = time (NULL);
-        file_system_db->root_directory = g_strdup (root_directory);
-        file_system_db->directory_names = directory_names;
-        file_system_db->file_names = file_names;
-        file_system_db->mime_types = medusa_hash_new
-                (mime_type_hash, MIME_HASH_BITS);
-        file_system_db->keyword_sets = medusa_hash_new
-                (keyword_sets_file_name, KEYWORD_SETS_HASH_BITS);
-
-        file_system_db->clauses = file_index_clauses ();
 
+        mime_type_list_name = medusa_generate_index_filename (MIME_TYPE_PREFIX,
+                                                              index_name,
+                                                              TRUE);
+        keyword_sets_list_name = medusa_generate_index_filename (KEYWORD_SETS_PREFIX,
+                                                              index_name,
+                                                              TRUE);
+        file_system_db->mime_types = medusa_string_list_new
+                (mime_type_list_name, MEDUSA_STRING_LIST_STRING_TO_KEY_WRITE_CACHE);
+        file_system_db->keyword_sets = medusa_string_list_new
+                (keyword_sets_list_name, MEDUSA_STRING_LIST_STRING_TO_KEY_WRITE_CACHE);
+        /* FIXME: No check whether the above operations (creating string lists) succeeded */
+        g_free (mime_type_list_name);
+        g_free (keyword_sets_list_name);
         file_system_db->keywords_by_file = g_hash_table_new
                 (g_str_hash, g_str_equal);
         file_system_db->private_keyword_users_by_directory = g_hash_table_new
@@ -161,23 +168,27 @@ medusa_file_system_db_new (const char *root_directory,
 
 
 MedusaFileSystemDB *
-medusa_file_system_db_open (const char *root_directory,
-                            const char *db_file_name,
-                            MedusaHash *file_names,
-                            MedusaHash *directory_names,
-                            const char *mime_type_hash,
-                            const char *keyword_sets_file_name)
+medusa_file_system_db_open (const char *index_name)
 {
         MedusaFileSystemDB *file_system_db;
         MedusaRDBFile *db_file;
+        char *attribute_index_filename, *mime_type_filename, *keyword_sets_filename;
         
-        g_return_val_if_fail (access (db_file_name, X_OK) != -1, NULL);
-        g_return_val_if_fail (access (keyword_sets_file_name, X_OK) != -1, NULL);
+
+
+        attribute_index_filename = medusa_generate_index_filename (FILE_ATTRIBUTE_DATABASE_PREFIX,
+                                                                   index_name,
+                                                                   FALSE);
+        db_file = medusa_rdb_file_open (attribute_index_filename, FILE_INDEX_METAINFO_LENGTH);
+        g_free (attribute_index_filename);
+
+        if (db_file == NULL) {
+                /* FIXME: bugzilla.eazel.com 4557: log error here */
+                g_warning ("Couldn't create file attribute index file\n");
+                return NULL;
+        }
 
         file_system_db = g_new0 (MedusaFileSystemDB, 1);
-
-        db_file = medusa_rdb_file_open (db_file_name, FILE_INDEX_METAINFO_LENGTH);
-
         
         /* Load the correct field info from the hard coded procedure */
         db_file->field_info = file_index_field_info ();
@@ -185,16 +196,34 @@ medusa_file_system_db_open (const char *root_directory,
         read_index_info_from_database (file_system_db);
         g_return_val_if_fail (file_system_db->version != NULL, NULL);
 
+        file_system_db->clauses_using_index_only = file_index_clauses_using_index_only ();
+        file_system_db->clauses_using_index_and_uid = file_index_clauses_using_index_and_uid ();
         file_system_db->indexing_start_time = time (NULL);
-        file_system_db->root_directory = g_strdup (root_directory);
-        file_system_db->directory_names = directory_names;
-        file_system_db->file_names = file_names;
-        file_system_db->mime_types = medusa_hash_open
-                (mime_type_hash, MIME_HASH_BITS);
-        file_system_db->keyword_sets = medusa_hash_open
-                (keyword_sets_file_name, KEYWORD_SETS_HASH_BITS);
 
-        file_system_db->clauses = file_index_clauses ();
+        mime_type_filename = medusa_generate_index_filename (MIME_TYPE_PREFIX,
+                                                             index_name,
+                                                             FALSE);
+        file_system_db->mime_types = medusa_string_list_open
+                (mime_type_filename, MEDUSA_STRING_LIST_CACHE_KEY_TO_STRING);
+        g_free (mime_type_filename);
+        if (file_system_db->mime_types == NULL) {
+                /* FIXME: bugzilla.eazel.com 4557: log error here */
+                g_warning ("Couldn't create mime type hash file\n");
+                goto cleanup;
+        }
+        
+        keyword_sets_filename = medusa_generate_index_filename (KEYWORD_SETS_PREFIX,
+                                                                index_name,
+                                                                FALSE);
+        file_system_db->keyword_sets = medusa_string_list_open
+                (keyword_sets_filename, MEDUSA_STRING_LIST_CACHE_KEY_TO_STRING);
+        g_free (keyword_sets_filename);
+        if (file_system_db->keyword_sets == NULL) {
+                /* FIXME: bugzilla.eazel.com 4557: log error here */
+                g_warning ("Couldn't create keyword hash file\n");
+                goto cleanup;
+        }       
+
 
         file_system_db->keywords_by_file = g_hash_table_new
                 (g_str_hash, g_str_equal);
@@ -202,9 +231,17 @@ medusa_file_system_db_open (const char *root_directory,
                 (g_str_hash, g_str_equal);
 
         return file_system_db;
+
+ cleanup:
+        if (file_system_db->mime_types != NULL) {
+                medusa_string_list_destroy (file_system_db->mime_types);
+        }
+        g_free (file_system_db);
+        return NULL;
 }
 
 
+
 int                  
 medusa_file_system_db_get_number_of_records (MedusaFileSystemDB *db)
 {
@@ -242,14 +279,12 @@ medusa_file_system_db_free (MedusaFileSystemDB *file_system_db)
         write_index_info_to_database (file_system_db,
                                       file_system_db->indexing_start_time);
         medusa_rdb_table_free (file_system_db->file_database);
-        g_free (file_system_db->root_directory);
         g_free (file_system_db->version);
 
-        medusa_hash_unref (file_system_db->directory_names);
-        medusa_hash_unref (file_system_db->file_names);
-        medusa_hash_unref (file_system_db->mime_types);
-        medusa_hash_unref (file_system_db->keyword_sets);
-        medusa_query_clauses_unref (file_system_db->clauses);
+        medusa_string_list_destroy (file_system_db->mime_types);
+        medusa_string_list_destroy (file_system_db->keyword_sets);
+        medusa_query_clauses_unref (file_system_db->clauses_using_index_only);
+        medusa_query_clauses_unref (file_system_db->clauses_using_index_and_uid);
         g_hash_table_foreach (file_system_db->keywords_by_file,
                               keywords_by_file_destroy_entry, NULL);
         g_hash_table_destroy (file_system_db->keywords_by_file);
@@ -275,6 +310,44 @@ medusa_file_system_db_index_file (MedusaFileSystemDB *file_system_db,
         medusa_file_attributes_free (file_attributes);
 }
 
+void                    
+medusa_file_system_db_erase_constructed_index (const char         *index_name)
+{
+        medusa_erase_constructed_index_file (FILE_ATTRIBUTE_DATABASE_PREFIX,
+                                             index_name);
+        medusa_erase_constructed_index_file (MIME_TYPE_PREFIX,
+                                             index_name);
+        medusa_erase_constructed_index_file (KEYWORD_SETS_PREFIX,
+                                             index_name);
+}
+
+void
+medusa_file_system_db_move_completed_index_into_place (const char     *index_name)
+{
+        medusa_move_completed_index_file_into_place (FILE_ATTRIBUTE_DATABASE_PREFIX,
+                                                     index_name);
+        medusa_move_completed_index_file_into_place (MIME_TYPE_PREFIX,
+                                                     index_name);
+        medusa_move_completed_index_file_into_place (KEYWORD_SETS_PREFIX,
+                                                     index_name);
+}
+
+gboolean                
+medusa_file_system_db_index_files_are_still_valid     (const char *index_name,
+                                                       time_t newest_valid_modified_time)
+{
+        return medusa_index_file_is_newer_than_time (FILE_ATTRIBUTE_DATABASE_PREFIX,
+                                                     index_name,
+                                                     newest_valid_modified_time) &&
+                medusa_index_file_is_newer_than_time (MIME_TYPE_PREFIX,
+                                                      index_name,
+                                                      newest_valid_modified_time) &&
+                medusa_index_file_is_newer_than_time (KEYWORD_SETS_PREFIX,
+                                                      index_name,
+                                                      newest_valid_modified_time);
+}
+
+
 static void
 write_index_info_to_database (MedusaFileSystemDB *file_system_db, int last_index_time)
 {
@@ -326,7 +399,7 @@ file_index_field_info (void)
 }
 
 static MedusaQueryClauses *
-file_index_clauses (void)
+file_index_clauses_using_index_only (void)
 {
         MedusaQueryClauses *clauses;
 
@@ -334,82 +407,99 @@ file_index_clauses (void)
         medusa_query_clauses_add_clause (clauses,
                                          "file_type",
                                          "is",
-                                         (MedusaQueryFunc) medusa_file_index_is_of_type,
+                                          medusa_file_index_is_of_type,
                                          MEDUSA_ARGUMENT_TYPE_STRING);
         medusa_query_clauses_add_clause (clauses,
                                          "file_type",
                                          "is_not",
-                                         (MedusaQueryFunc) medusa_file_index_is_not_of_type,
+                                          medusa_file_index_is_not_of_type,
                                          MEDUSA_ARGUMENT_TYPE_STRING);
-        medusa_query_clauses_add_clause (clauses,
-                                         "mtime",
-                                         "is_before",
-                                         (MedusaQueryFunc) medusa_file_index_is_modified_before_time,
-                                         MEDUSA_ARGUMENT_TYPE_INTEGER);
-        medusa_query_clauses_add_clause (clauses,
-                                         "mtime",
-                                         "is_after",
-                                         (MedusaQueryFunc) medusa_file_index_is_modified_after_time,
-                                         MEDUSA_ARGUMENT_TYPE_INTEGER);
+        medusa_query_clauses_add_clause_with_check (clauses,
+                                                    "mtime",
+                                                    "is_before",
+                                                     medusa_file_index_is_modified_before_time,
+                                                    medusa_file_index_verify_file_is_modified_before_time,
+                                                    MEDUSA_ARGUMENT_TYPE_NUMBER);
+        medusa_query_clauses_add_clause_with_check (clauses,
+                                                    "mtime",
+                                                    "is_after",
+                                                     medusa_file_index_is_modified_after_time,
+                                                    medusa_file_index_verify_file_is_modified_after_time,
+                                                    MEDUSA_ARGUMENT_TYPE_NUMBER);
         medusa_query_clauses_add_clause (clauses,
                                          "owner",
                                          "has_uid",
-                                         (MedusaQueryFunc) medusa_file_index_has_uid,
-                                         MEDUSA_ARGUMENT_TYPE_INTEGER);
+                                          medusa_file_index_has_uid,
+                                         MEDUSA_ARGUMENT_TYPE_NUMBER);
         medusa_query_clauses_add_clause (clauses,
                                          "owner",
                                          "does_not_have_uid",
-                                         (MedusaQueryFunc) medusa_file_index_does_not_have_uid,
-                                         MEDUSA_ARGUMENT_TYPE_INTEGER);
+                                          medusa_file_index_does_not_have_uid,
+                                         MEDUSA_ARGUMENT_TYPE_NUMBER);
         medusa_query_clauses_add_clause (clauses,
                                          "group",
                                          "has_gid",
-                                         (MedusaQueryFunc) medusa_file_index_has_gid,
-                                         MEDUSA_ARGUMENT_TYPE_INTEGER);
+                                          medusa_file_index_has_gid,
+                                         MEDUSA_ARGUMENT_TYPE_NUMBER);
         medusa_query_clauses_add_clause (clauses,
                                          "group",
                                          "does_not_have_gid",
-                                         (MedusaQueryFunc) medusa_file_index_does_not_have_gid,
-                                         MEDUSA_ARGUMENT_TYPE_INTEGER);
+                                          medusa_file_index_does_not_have_gid,
+                                         MEDUSA_ARGUMENT_TYPE_NUMBER);
         medusa_query_clauses_add_clause (clauses,
                                          "size",
                                          "larger_than",
-                                         (MedusaQueryFunc) medusa_file_index_is_larger_than,
-                                         MEDUSA_ARGUMENT_TYPE_INTEGER);
+                                          medusa_file_index_is_larger_than,
+                                         MEDUSA_ARGUMENT_TYPE_NUMBER);
         medusa_query_clauses_add_clause (clauses,
                                          "size",
                                          "smaller_than",
-                                         (MedusaQueryFunc) medusa_file_index_is_smaller_than,
-                                         MEDUSA_ARGUMENT_TYPE_INTEGER);
+                                          medusa_file_index_is_smaller_than,
+                                         MEDUSA_ARGUMENT_TYPE_NUMBER);
         medusa_query_clauses_add_clause (clauses,
                                          "size",
                                          "is",
-                                         (MedusaQueryFunc) medusa_file_index_is_of_size,
-                                         MEDUSA_ARGUMENT_TYPE_INTEGER);
+                                          medusa_file_index_is_of_size,
+                                         MEDUSA_ARGUMENT_TYPE_NUMBER);
         medusa_query_clauses_add_clause (clauses,
                                          "permissions_to_read",
                                          "include_uid",
-                                         (MedusaQueryFunc) medusa_file_index_uid_can_read_file,
-                                         MEDUSA_ARGUMENT_TYPE_INTEGER);
+                                          medusa_file_index_uid_can_read_file,
+                                         MEDUSA_ARGUMENT_TYPE_NUMBER);
+
+        return clauses;
+}
+
+static MedusaQueryClauses *
+file_index_clauses_using_index_and_uid (void)
+{
+        MedusaQueryClauses *clauses;
+
+        clauses = medusa_query_clauses_new ();
         medusa_query_clauses_add_clause (clauses,
                                          "keywords",
                                          "include",
-                                         (MedusaQueryFunc) medusa_file_index_marked_with_keyword,
+                                          medusa_file_index_marked_with_keyword,
                                          MEDUSA_ARGUMENT_TYPE_STRING);
         medusa_query_clauses_add_clause (clauses,
                                          "keywords",
                                          "do_not_include",
-                                         (MedusaQueryFunc) medusa_file_index_not_marked_with_keyword,
+                                          medusa_file_index_not_marked_with_keyword,
                                          MEDUSA_ARGUMENT_TYPE_STRING);
-
         return clauses;
 }
 
 
 MedusaQueryClauses *
-medusa_file_system_db_get_query_clauses (MedusaFileSystemDB *db)
+medusa_file_system_db_get_query_clauses_using_index_only (MedusaFileSystemDB *db)
 {
-        return db->clauses;
+        return db->clauses_using_index_only;
+}
+
+MedusaQueryClauses *
+medusa_file_system_db_get_query_clauses_using_index_and_uid (MedusaFileSystemDB *db)
+{
+        return db->clauses_using_index_and_uid;
 }
 
 static MedusaKeywordSet *
@@ -551,3 +641,4 @@ medusa_file_system_db_update_keywords (MedusaFileSystemDB *db,
                 g_free (keyword_set_as_string);
         }
 }
+
