@@ -110,7 +110,7 @@ medusa_io_handler_open (const char *file_name,
 
         file_descriptor = open (file_name, O_RDWR);        
         if (file_descriptor == -1) {
-                /* FIXME: Log this event */
+                /* FIXME bugzilla.eazel.com 4557: Log this event */
                 return NULL;
         }
 
@@ -120,7 +120,7 @@ medusa_io_handler_open (const char *file_name,
         handler->magic_number = g_strdup (magic_number);
         handler->version = g_strdup (version_number);
 
-        lstat (file_name, &file_info);
+        stat (file_name, &file_info);
         handler->file_size = file_info.st_size;
         medusa_io_handler_read_file_header (handler);
         
@@ -149,10 +149,10 @@ medusa_io_handler_free (MedusaIOHandler *handler)
                                         handler->version,
                                         handler->buffer_space_available);
         if  (munmap (handler->mapped_region, handler->mapped_size) == -1) {
-                /* FIXME: Log error */
+                /* FIXME bugzilla.eazel.com 4557: Log error */
         }
         if (close (handler->file_descriptor) == -1) {
-                /* FIXME: log error */
+                /* FIXME bugzilla.eazel.com 4557: log error */
         }
         g_free (handler->magic_number);
         g_free (handler->version);
@@ -241,8 +241,6 @@ fopen_new_with_medusa_io_handler_header (const char *file_name,
         FILE *fp;
         struct stat file_info;
 
-        /* Don't deal with files that exist already, tehis will
-           seriously mess them up */
         /* FIXME bugzilla.eazel.com 2696: 
            Perhaps we should do a check here,
            and if the file exists and is a medusa io
