@@ -27,6 +27,7 @@
 #ifndef MEDUSA_LOCK_FILE_PATHS_H
 #define MEDUSA_LOCK_FILE_PATHS_H
 
-#define MEDUSA_FILE_INDEX_LOCK_FILE "/var/lock/medusa-db"
+#define MEDUSA_FILE_INDEX_READ_LOCK_FILE "/var/lock/medusa-db-read"
+#define MEDUSA_FILE_INDEX_WRITE_LOCK_FILE "/var/lock/medusa-db-write"
 
 #endif /* MEDUSA_LOCK_FILE_PATHS_H */
