@@ -39,11 +39,15 @@
 #include <unistd.h>
 
 #include <libmedusa/medusa-index-progress.h>
+#include <libmedusa/medusa-log.h>
 #include <libmedusa/medusa-unsearched-locations.h>
+#include <libmedusa/medusa-string.h>
 #include <libmedusa/medusa-utils.h>
+#include <medusa-idled/medusa-idled-client.h>
 
 #include "medusa-conf.h"
 #include "medusa-file-index.h"
+#include "medusa-index-filenames.h"
 #include "medusa-master-db-private.h"
 #include "medusa-rdb-file.h"
 #include "medusa-rdb-table.h"
@@ -87,13 +91,14 @@ static GList *               run_query_on_all_uris                        (Medus
 						       
 /* Run a set of queries on a uri, and if the uri fits, add it to the current results */
 static GList *               append_uri_to_results_if_matches             (GList                *result_list,
-									   GList                *clause_closures,
+									   GList                *parsed_search_criteria,
 									   MedusaMasterDB       *master_db,
 									   int                   uri_number);
-static gboolean              query_execute_from_clause_closure            (MedusaClauseClosure  *clause_closure,
+static gboolean              query_execute_from_parsed_search_criterion            (MedusaParsedSearchCriterion  *parsed_search_criterion,
+										    MedusaMasterDB *master_db,
 									   int                   uri_number);
 static MedusaContentResults *content_requests_to_uri_numbers              (MedusaMasterDB       *master_db,
-									   GList                *clause_closures);
+									   GList                *parsed_search_criteria);
 static void                  merge_new_uris_with_current_results_and_free (MedusaContentResults *old_results,
 									   gint32               *new_uris,
 									   int                   number_of_new_uris,
@@ -105,112 +110,99 @@ static void                  medusa_master_db_destroy                     (Medus
 
 MedusaMasterDB *
 medusa_master_db_open (const char     *root_uri,
-		       MedusaLogLevel  log_level,
-		       const char     *uri_list,
-		       const char     *file_attribute_index,
-		       const char     *file_name_hash,
-		       const char     *directory_name_hash,
-		       const char     *mime_type_hash,
-		       const char     *keyword_sets_file_name,
-		       const char     *text_index_start_file,
-		       const char     *text_index_location_file,
-		       const char     *text_index_word_file)
+		       const char     *index_name,
+		       MedusaLogLevel  log_level)
 {
 	MedusaMasterDB *master_db;
-	MedusaHash *file_names, *directory_names;
 
-	/* Add check for all files existing here */
+	g_return_val_if_fail (root_uri != NULL, NULL);
+	g_return_val_if_fail (index_name != NULL, NULL);
 
 	master_db = g_new0 (MedusaMasterDB, 1);
-	file_names = medusa_hash_open (file_name_hash,
-				       FILE_HASH_BITS);
-
-	directory_names = medusa_hash_open (directory_name_hash,
-					    DIRECTORY_HASH_BITS);
-	if (file_names == NULL || directory_names == NULL) {
-		g_free (master_db);
-		/* FIXME: log error here */
+	master_db->uri_list = medusa_uri_list_open (index_name,
+						    log_level);
+	if (master_db->uri_list == NULL) {
+		/* FIXME bugzilla.eazel.com 4557: log error here */
 		return NULL;
 	}
-	master_db->uri_list = medusa_uri_list_open (root_uri, log_level, uri_list, file_names, directory_names);
-	g_return_val_if_fail (master_db->uri_list != NULL, NULL);
-	master_db->text_index = medusa_text_index_open (text_index_start_file,
-							log_level,
-							text_index_location_file,
-							text_index_word_file);
-	g_return_val_if_fail (master_db->text_index != NULL, NULL);
-	master_db->file_system_db = medusa_file_system_db_open
-		(ROOT_DIRECTORY, file_attribute_index,
-		 file_names, directory_names,
-		 mime_type_hash, keyword_sets_file_name);
-	g_return_val_if_fail (master_db->file_system_db != NULL, NULL);
-	medusa_hash_ref (file_names);
-	medusa_hash_ref (directory_names);
-	master_db->root_uri = g_strdup (root_uri);
+
+	master_db->text_index = medusa_text_index_open (index_name,
+							log_level);
 	
+	if (master_db->text_index == NULL) {
+		/* FIXME bugzilla.eazel.com 4557: log error here */
+		goto cleanup;
+	}
+
+	master_db->file_system_db = medusa_file_system_db_open (index_name);
+	
+	if (master_db->file_system_db == NULL) {
+		/* FIXME bugzilla.eazel.com 4557: log error here */
+		goto cleanup;
+	}
+
+	master_db->root_uri = g_strdup (root_uri);
 	master_db->log_level = log_level;
 	master_db->indexing_mode = FALSE;
 	master_db->ref_count = 1;
+
 	return master_db;
 
+ cleanup:
+	if (master_db == NULL) {
+		return NULL;
+	}
+	if (master_db->uri_list != NULL) {
+		medusa_uri_list_unref (master_db->uri_list);
+	}
+	if (master_db->file_system_db != NULL) {
+		medusa_file_system_db_free (master_db->file_system_db);
+	}
+	if (master_db->text_index != NULL) {
+		medusa_text_index_unref (master_db->text_index);
+	}
+	g_free (master_db);
+	return NULL;
+	
 }
 
-MedusaMasterDB *
-medusa_master_db_new (const char *root_uri,
-		      MedusaLogLevel log_level,
-		      const char *uri_list,
-		      const char *file_attribute_index,
-		      const char *file_name_hash,
-		      const char *directory_name_hash,
-		      const char *mime_type_hash,
-		      const char *keyword_sets_file_name,
-		      const char *text_index_start_file,
-		      const char *text_index_location_file,
-		      const char *text_index_word_file,
-		      const char *text_index_temp_file)
+void
+medusa_master_db_create_index (const char *root_uri,
+			       const char *index_name,
+			       gboolean dont_use_idled_service,
+			       MedusaLogLevel log_level)
 {
 	MedusaMasterDB *master_db;
-	MedusaHash *file_names, *directory_names;
 	char *root_directory;
 
+	g_return_if_fail (root_uri != NULL);
+	g_return_if_fail (index_name != NULL);
+
 	master_db = g_new0 (MedusaMasterDB, 1);
-  
 
-	file_names = medusa_hash_new (file_name_hash,
-				      FILE_HASH_BITS);
-	directory_names = medusa_hash_new (directory_name_hash,
-					   DIRECTORY_HASH_BITS);
 	root_directory = gnome_vfs_get_local_path_from_uri (root_uri);
-	master_db->indexing_mode = TRUE;
+	master_db->use_idle_service = !dont_use_idled_service;
 	master_db->index_progress = medusa_index_progress_new (root_directory);
-	master_db->uri_list = medusa_uri_list_new (root_uri, log_level, uri_list, file_names, directory_names);
-	master_db->text_index = medusa_text_index_new (text_index_start_file,
-						       log_level,
-						       text_index_location_file,
-						       text_index_word_file,
-						       text_index_temp_file);
+	if (master_db->use_idle_service) {
+		medusa_idle_service_register (&master_db->idled_connection);
+		medusa_idle_service_sleep_until_idle (master_db->idled_connection);
+	}
+	master_db->indexing_mode = TRUE;
 
-	master_db->file_system_db = medusa_file_system_db_new
-		(root_directory, file_attribute_index,
-		 file_names, directory_names,
-		 mime_type_hash, keyword_sets_file_name);
+	master_db->uri_list = medusa_uri_list_new (index_name, 
+						   log_level);
+	master_db->text_index = medusa_text_index_new (index_name,
+						       log_level);
+	master_db->file_system_db = medusa_file_system_db_new (index_name);
 	g_free (root_directory);
 	
-	medusa_hash_ref (file_names);
-	medusa_hash_ref (directory_names);
 	master_db->root_uri = g_strdup (root_uri);
 	
 	master_db->log_level = log_level;
-
 	master_db->ref_count = 1;
 
 	medusa_unsearched_locations_initialize ();
-	return master_db;
-}	      
 
-void
-medusa_master_db_index (MedusaMasterDB *master_db)
-{
 	/* Make sure that the databases are empty before we start */
 	g_assert (medusa_uri_list_number_of_uris (master_db->uri_list) == 0);
 	g_assert (medusa_file_system_db_get_number_of_records (master_db->file_system_db) == 0);
@@ -218,47 +210,47 @@ medusa_master_db_index (MedusaMasterDB *master_db)
 	index_private_metafiles (master_db);
 	index_directory (master_db, master_db->root_uri);
 #ifdef TEXT_INDEX_ON
-	medusa_text_index_finish_indexing (master_db->text_index);
+	medusa_text_index_finish_indexing (master_db->text_index,
+					   master_db->use_idle_service,
+					   master_db->idled_connection);
 #endif
 	medusa_file_system_db_update_keywords (master_db->file_system_db,
 					       master_db->uri_list);
+
+	if (master_db->use_idle_service) {
+		medusa_idle_service_unregister_and_destroy_connection (master_db->idled_connection);
+	}
+	medusa_master_db_unref (master_db);
+	medusa_unsearched_locations_shutdown ();
+
 }
 
-
-void
-medusa_master_db_update (MedusaMasterDB *master_db)
+gboolean
+medusa_master_db_index_files_are_still_valid (const char *index_name,
+					      time_t newest_valid_modified_time)
 {
-	/* FIXME bugzilla.eazel.com 2713: 
-	   Not implemented. */
+	return medusa_file_system_db_index_files_are_still_valid (index_name,
+								   newest_valid_modified_time) &&
+		medusa_uri_list_index_files_are_still_valid (index_name,
+							     newest_valid_modified_time) &&
+		medusa_text_index_files_are_still_valid (index_name,
+							 newest_valid_modified_time);
 }
 
+
 /* Returns a glist of uri's */
-/* FIXME:  Return errors, too */
+/* FIXME bugzilla.eazel.com 4558:  Return errors, too */
 GList *
-medusa_master_db_query (MedusaMasterDB *master_db,
-			char *search_uri_string)
+medusa_master_db_run_search (MedusaMasterDB *master_db,
+			     MedusaParsedSearchURI *parsed_search_uri)
 {
-	MedusaParsedSearchURI *parsed_search_uri;
+
 	GList *results;
 	MedusaContentResults *content_results;
 	GList *content_clauses, *non_content_clauses;
 
-
-	parsed_search_uri = medusa_search_uri_parse (search_uri_string, master_db);
-	if (parsed_search_uri->syntax_error_found ||
-	    parsed_search_uri->query_is_always_false) {
-		medusa_parsed_search_uri_free (parsed_search_uri);
-		return NULL;
-	}
-	/* FIXME: What do we do if the query is always true? */
-	/* Return nothing for uris that are invalid */
-	if (parsed_search_uri->clause_closures == NULL) {
-		medusa_parsed_search_uri_free (parsed_search_uri);
-		return NULL;
-	}
-
-	content_clauses = medusa_g_list_partition (parsed_search_uri->clause_closures,
-						   medusa_clause_closure_is_content_search,
+	content_clauses = medusa_g_list_partition (parsed_search_uri->parsed_search_criteria,
+						   medusa_parsed_search_criterion_is_content_search,
 						   NULL,
 						   &non_content_clauses);
 	if (content_clauses != NULL) {
@@ -275,7 +267,7 @@ medusa_master_db_query (MedusaMasterDB *master_db,
 									       non_content_clauses,
 									       content_results);
 		}
-
+		g_free (content_results);
 	}
 	else {
 		results = run_query_on_all_uris (master_db,
@@ -284,34 +276,66 @@ medusa_master_db_query (MedusaMasterDB *master_db,
 	/* g_list_partition consumed the original list,
 	   and so we must free the clause closures out of content_clauses
 	   and non_content_closures instead of out of the parsed search uri */
-	medusa_clause_closure_list_free (content_clauses);
-	medusa_clause_closure_list_free (non_content_clauses);
-	medusa_parsed_search_uri_free (parsed_search_uri);
+	medusa_parsed_search_criterion_list_free (content_clauses);
+	medusa_parsed_search_criterion_list_free (non_content_clauses);
+
 	return results;
 
 }
+
+void            
+medusa_master_db_erase_constructed_index (const char *index_name)
+{
+
+	medusa_file_system_db_erase_constructed_index (index_name);
+	medusa_uri_list_erase_constructed_index (index_name);
+	medusa_text_index_erase_constructed_index (index_name);
+}
 			
+void
+medusa_master_db_move_completed_index_into_place (const char *index_name)
+{
+	medusa_file_system_db_move_completed_index_into_place (index_name);
+	medusa_uri_list_move_completed_index_into_place (index_name);
+	medusa_text_index_move_completed_index_into_place (index_name);
+}
 
 static gboolean
-query_execute_from_clause_closure (MedusaClauseClosure *clause_closure,
-				   int uri_number)
+query_execute_from_parsed_search_criterion (MedusaParsedSearchCriterion *parsed_search_criterion,
+					    MedusaMasterDB *master_db,
+					    int uri_number)
 {
-	g_return_val_if_fail (clause_closure->query_func != NULL, FALSE);
-	if (clause_closure->file_system_db != NULL) {
-		return clause_closure->query_func (clause_closure->file_system_db,
-						   medusa_rdb_record_number_to_record (clause_closure->file_system_db->file_database,
-										       uri_number),
-						   clause_closure->argument);
-	}
-	else {
-		return clause_closure->query_func (clause_closure->uri_list,
-						   medusa_rdb_record_number_to_record (clause_closure->uri_list->uri_names,
-										       uri_number),
-						   clause_closure->argument);
+	MedusaRDBRecord database_record;
+	
+	g_return_val_if_fail (parsed_search_criterion->query_func != NULL, FALSE);
+
+	switch (parsed_search_criterion->database_record_type) {
+	case MEDUSA_FILE_ATTRIBUTE_INDEX_RECORD:
+		database_record = medusa_rdb_record_number_to_record (master_db->file_system_db->file_database,
+								      uri_number);
+		break;
+	case MEDUSA_URI_INDEX_RECORD:
+		database_record = medusa_rdb_record_number_to_record (master_db->uri_list->uri_names,
+								      uri_number);
+		break;
+	default:
+		g_assert_not_reached ();
+		database_record = NULL;
 	}
+	return parsed_search_criterion->query_func (parsed_search_criterion->query_data,
+						    database_record,
+						    parsed_search_criterion->argument);
 }
 				
 				       
+static gboolean
+file_name_looks_invalid (const char *file_name)
+{
+	/* Files of length 0 are no good, since
+	   they can't be differentiated from
+	   the directory that contains them. */
+	return (file_name == NULL || file_name[0] == 0);
+}
 
 static void
 index_directory (MedusaMasterDB *master_db,
@@ -321,6 +345,7 @@ index_directory (MedusaMasterDB *master_db,
 	char *directory_path;
 	struct dirent *directory_entry;
 	char *file_uri;
+	char *log_message;
 	GnomeVFSFileInfo *file_info;
 	GnomeVFSResult result;
   
@@ -333,21 +358,22 @@ index_directory (MedusaMasterDB *master_db,
 		g_free (directory_path);
 		return;
 	}
-	
+	if (master_db->use_idle_service) {
+		medusa_idle_service_sleep_until_idle (master_db->idled_connection);
+	}
 	while ((directory_entry = readdir (directory_handle))) {
 		if (!strcmp (directory_entry->d_name,".") || 
-		    !strcmp (directory_entry->d_name,"..")) {
+		    !strcmp (directory_entry->d_name,"..") ||
+		    file_name_looks_invalid (directory_entry->d_name)) {
 			continue;
 		}
 		
 		file_uri = medusa_full_uri_from_directory_and_file (directory_uri,
 								    directory_entry->d_name);
-		
 		if (medusa_is_unsearched_location (file_uri)) {
-			if (master_db->log_level == MEDUSA_DB_LOG_ABBREVIATED ||
-			    master_db->log_level == MEDUSA_DB_LOG_EVERYTHING) {
-				printf ("Skipping file %s\n", file_uri);
-			}
+			log_message = g_strdup_printf ("Skipping file %s because it is on the stop list", file_uri);
+			medusa_log_event (log_message, MEDUSA_DB_LOG_ABBREVIATED);
+			g_free (log_message);
 			g_free (file_uri);
 			continue;
 		}
@@ -360,11 +386,18 @@ index_directory (MedusaMasterDB *master_db,
 			g_free (file_uri);
 			continue;
 		}
-
+		log_message = g_strdup_printf ("Indexing file %s in directory uri %s", directory_entry->d_name, directory_uri);
+		medusa_log_event (log_message,
+				  MEDUSA_DB_LOG_ABBREVIATED);
+		g_free (log_message);
 		index_file (master_db, directory_uri, directory_entry->d_name, file_info);
 
 		if (file_info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
 			gnome_vfs_file_info_unref (file_info);
+			log_message = g_strdup_printf ("Indexing directory uri %s", file_uri);
+			medusa_log_event (log_message,
+					  MEDUSA_DB_LOG_ABBREVIATED);
+			g_free (log_message);
 			index_directory (master_db, file_uri);
 		}
 		else {
@@ -393,16 +426,11 @@ index_file (MedusaMasterDB *master_db,
 	
 	uri_number = medusa_uri_list_number_of_uris (master_db->uri_list);
 	medusa_uri_list_index_file (master_db->uri_list, directory_uri, file_name);
-	uri = medusa_uri_number_to_uri (master_db->uri_list, uri_number);
-	if (master_db->log_level == MEDUSA_DB_LOG_EVERYTHING) {
-		printf ("Indexing name and attributes of %s\n",uri);
-	}
-	if (master_db->log_level == MEDUSA_DB_LOG_ABBREVIATED) {
-		printf ("%s\n",uri);
-	}
 	medusa_file_system_db_index_file (master_db->file_system_db, 
 					  uri_number,
 					  file_info);
+	uri = medusa_full_uri_from_directory_and_file (directory_uri,
+						       file_name);
 #ifdef TEXT_INDEX_ON
 	medusa_text_index_read_file (master_db->text_index,
 				     uri,
@@ -478,48 +506,73 @@ run_query_on_all_uris (MedusaMasterDB *master_db,
 	return results;
 }
 
+static gboolean
+criterion_check_is_true_for_uri_if_check_exists (gpointer data,
+						 gpointer user_data)
+{
+	char *uri;
+	MedusaParsedSearchCriterion *criterion;
+
+	uri = (char *) user_data;
+	criterion = (MedusaParsedSearchCriterion *) data;
+
+	if (!medusa_parsed_search_criterion_has_check (criterion)) {
+		return TRUE;
+	}
+	else {
+		return medusa_parsed_search_criterion_run_check (criterion,
+							uri);
+	}
+}
+			
 
 static GList *     
 append_uri_to_results_if_matches (GList *result_list,
-				  GList *clause_closures,
+				  GList *parsed_search_criteria,
 				  MedusaMasterDB *master_db,
 				  int uri_number)
 {
 	gboolean query_result;
 	GList *next_clause;
+	char *uri;
 	
 	query_result = TRUE;
 	/* Need two cases: the first is for non-partitioned lists, the second for 
 	   parititioned lists */
-	for (next_clause = clause_closures; next_clause != NULL && next_clause->data != NULL; next_clause = next_clause->next) {
+	for (next_clause = parsed_search_criteria; next_clause != NULL && next_clause->data != NULL; next_clause = next_clause->next) {
 		
-		if (query_execute_from_clause_closure (next_clause->data, uri_number) == FALSE) {
+		if (query_execute_from_parsed_search_criterion (next_clause->data, master_db, uri_number) == FALSE) {
 			query_result = FALSE;
 			break;
 		}
 	}
 	if (query_result) {
-		result_list = g_list_prepend (result_list, 
-					      medusa_uri_number_to_uri (master_db->uri_list,
-									uri_number));
+		uri = medusa_uri_number_to_uri (master_db->uri_list,
+						uri_number);
+		if (medusa_g_list_forall (parsed_search_criteria, 
+					  criterion_check_is_true_for_uri_if_check_exists, uri)) {
+			
+			result_list = g_list_prepend (result_list, 
+						      uri);
+		}
 	}
 	return result_list;
-		
+	
 }
 
 static MedusaContentResults *    
 content_requests_to_uri_numbers (MedusaMasterDB *master_db,
-				 GList *clause_closures)
+				 GList *parsed_search_criteria)
 {
-	MedusaClauseClosure *closure, *new_closure_data;
+	MedusaParsedSearchCriterion *closure, *new_closure_data;
 	GList *new_closure;
 	gint32 *new_uris;
 	int number_of_new_uris;
 	MedusaContentResults *results;
 	
 
-	g_return_val_if_fail (g_list_length (clause_closures) > 0, NULL);
-	closure = clause_closures->data;
+	g_return_val_if_fail (g_list_length (parsed_search_criteria) > 0, NULL);
+	closure = parsed_search_criteria->data;
 	g_assert (closure->is_content_request == TRUE &&
 		  (closure->return_matches == TRUE ||
 		   closure->return_matches == FALSE));
@@ -530,14 +583,14 @@ content_requests_to_uri_numbers (MedusaMasterDB *master_db,
 								  closure->match_all_words,
 								  &results->number_of_uris);
 	/* No intersection work to do if there is only one criterion */
-	if (g_list_length (clause_closures) == 1) {
+	if (g_list_length (parsed_search_criteria) == 1) {
 		return results;
 	}
 	/* Otherwise, keep intersecting the results with the new results */
-	for (new_closure = clause_closures->next; 
+	for (new_closure = parsed_search_criteria->next; 
 	     new_closure != NULL; 
 	     new_closure = new_closure->next) {
-		new_closure_data = (MedusaClauseClosure *) new_closure->data;
+		new_closure_data = (MedusaParsedSearchCriterion *) new_closure->data;
 		new_uris = medusa_text_index_get_uris_for_criterion (master_db->text_index,
 								     new_closure_data->content_request,
 								     new_closure_data->match_all_words,
@@ -641,11 +694,10 @@ medusa_master_db_destroy (MedusaMasterDB *master_db)
 	medusa_uri_list_unref (master_db->uri_list);
 	medusa_file_system_db_free (master_db->file_system_db);
 	medusa_text_index_unref (master_db->text_index);
-	medusa_unsearched_locations_shutdown ();
 	if (master_db->indexing_mode) {
 		medusa_index_progress_destroy (master_db->index_progress);
 	}
-	medusa_search_uri_parse_shutdown ();
+	medusa_search_uri_clean_up_parse_data ();
 	g_free (master_db->root_uri);
 	g_free (master_db);
 }
@@ -836,3 +888,4 @@ index_private_metafiles (MedusaMasterDB *db)
 	endpwent ();
 	g_free (user);
 }
+
