@@ -22,10 +22,14 @@
  *  Author: Rebecca Schulman <rebecka@eazel.com>
  */
 
-/* medusa-query-clauses.c:  System for creating a
-   vocabulary for medusa databases (phrases that correspond to
-   a function that can find things that satisfy a phrase) */
+/* medusa-query-clauses.c: Each database registers the queries it knows
+   how to perform as a set of query clauses, that are then registered with
+   the top level database */
 
+/* FIXME bugzilla.eazel.com 4885: Rename this class to make it more comprehensible, and less confusing
+   w.r.t. the clause closures */
+
+#include <ctype.h>
 #include <glib.h>
 #include <string.h>
 
@@ -45,17 +49,26 @@ struct MedusaQueryClause {
 	char *operator_name;
 	MedusaQueryFunc evaluate;
 	MedusaArgumentType argument_type;	
+	/* Function that verifies that the real file 
+	   still matches the criteria.  For now we assume it
+	takes the same type as the evaluate function*/
+	MedusaResultCheckFunc verify;
 	int ref_count;
 };
 
 
-
+/* FIXME bugzilla.eazel.com 4886: Get rid of ref counting here; it is unnecessary */
 static void                    medusa_query_clauses_destroy             (MedusaQueryClauses *clause);
 
-static MedusaQueryClause *     medusa_query_clause_new                  (char *field_name,
-									 char *operator_name,
+static MedusaQueryClause *     medusa_query_clause_new                  (const char *field_name,
+									 const char *operator_name,
 									 MedusaQueryFunc evaluate,
 									 MedusaArgumentType argument_type);
+static MedusaQueryClause *     medusa_query_clause_new_with_check       (const char *field_name,
+									 const char *operator_name,
+									 MedusaQueryFunc evaluate,
+									 MedusaResultCheckFunc verify,
+									 MedusaArgumentType argument_type);
 /*static void                    medusa_query_clause_ref                  (MedusaQueryClause *clause); */
 static void                    medusa_query_clause_unref                (MedusaQueryClause *clause);
 static void                    medusa_query_clause_destroy              (MedusaQueryClause *clause);
@@ -71,6 +84,25 @@ medusa_query_clauses_new (void)
 	return clauses;
 }
 
+void
+medusa_query_clauses_add_clause_with_check (MedusaQueryClauses *clauses,
+					    const char *field_name,
+					    const char *operator_name,
+					    MedusaQueryFunc evaluate,
+					    MedusaResultCheckFunc verify,
+					    MedusaArgumentType argument_type)
+{
+	MedusaQueryClause *new_clause;
+
+	new_clause = medusa_query_clause_new_with_check (field_name, 
+							 operator_name, 
+							 evaluate, 
+							 verify,
+							 argument_type);
+	clauses->clauses = g_list_prepend (clauses->clauses, new_clause);
+}
+					    
+
 void
 medusa_query_clauses_add_clause (MedusaQueryClauses *clauses,
 				 char *field_name,
@@ -84,31 +116,85 @@ medusa_query_clauses_add_clause (MedusaQueryClauses *clauses,
 	clauses->clauses = g_list_prepend (clauses->clauses, new_clause);
 }
 
-MedusaQueryFunc
-medusa_query_clauses_get_function (MedusaQueryClauses *clauses,
-				   const char *clause,
-				   MedusaArgumentType *argument_type)
+static gboolean
+value_has_correct_type (MedusaArgumentType argument_type,
+			const char *value_to_check)
 {
+	const char *arg;
+
+	switch (argument_type) {
+	case MEDUSA_ARGUMENT_TYPE_NONE:
+		return value_to_check == NULL ||
+			strlen (value_to_check) == 0;
+	case MEDUSA_ARGUMENT_TYPE_NUMBER:
+		for (arg = value_to_check; *arg != 0; arg++) {
+			/* We could use a better check than this */
+			if (!isdigit (*arg)) {
+				g_print ("Value doesn't have type integer.  returning false\n");
+				return FALSE;
+			}
+		}
+		return TRUE;
+	case MEDUSA_ARGUMENT_TYPE_STRING:
+		return TRUE;
+	default:
+		g_assert_not_reached ();
+		return FALSE;
+	}
+		
+}
+
+static MedusaQueryClause *
+match_search_clause_to_query_clause (MedusaQueryClauses *clauses,
+				     const char *clause)
+{
+	char **words, *field_name, *operator_name, *value;
 	GList *i;
-	char **words, *field_name, *operator_name;
+	MedusaQueryClause *query_clause;
 	
 	words = g_strsplit (clause, " ", 3);
 	field_name = words[0];
 	operator_name = words[1];
-	
+	value = words[2];
+
 	for (i = clauses->clauses; i != NULL; i = i->next) {
-		if (strcmp (((MedusaQueryClause *) i->data)->field_name, field_name) == 0 &&
-		    strcmp (((MedusaQueryClause *) i->data)->operator_name, operator_name) == 0) {
-			*argument_type = ((MedusaQueryClause *) i->data)->argument_type;
+		query_clause = (MedusaQueryClause *) i->data;
+		if (strcmp (query_clause->field_name, field_name) == 0 &&
+		    strcmp (query_clause->operator_name, operator_name) == 0 &&
+		    value_has_correct_type (query_clause->argument_type, value)) {
 			g_strfreev (words);
-			return ((MedusaQueryClause *) i->data)->evaluate;
+			return i->data;
 		}
 	}
-
 	g_strfreev (words);
 	return NULL;
 }
 
+void	
+medusa_query_clauses_get_function (MedusaQueryClauses *clauses,
+				   const char *clause,
+				   MedusaQueryFunc *evaluate,
+				   MedusaResultCheckFunc *verify,
+				   MedusaArgumentType *argument_type)
+{
+	MedusaQueryClause *matching_function;
+
+	matching_function = match_search_clause_to_query_clause (clauses,
+								 clause);
+
+	if (matching_function == NULL) {
+		*evaluate = NULL;
+		*verify = NULL;
+		return;
+	}
+	else {
+		*evaluate = matching_function->evaluate;
+		*verify = matching_function->verify;
+		*argument_type = matching_function->argument_type;
+	}
+
+}
+
 void
 medusa_query_clauses_ref (MedusaQueryClauses *clauses)
 {
@@ -119,6 +205,9 @@ medusa_query_clauses_ref (MedusaQueryClauses *clauses)
 void
 medusa_query_clauses_unref (MedusaQueryClauses *clauses)
 {
+	if (clauses == NULL) {
+		return;
+	}
 	g_assert (clauses->ref_count > 0);
 	if (clauses->ref_count == 1) {
 		medusa_query_clauses_destroy (clauses);
@@ -129,37 +218,6 @@ medusa_query_clauses_unref (MedusaQueryClauses *clauses)
 
 
 
-gboolean
-medusa_query_result_succeeded (MedusaQueryResult *result)
-{
-	return result->result;
-}
-
-gpointer
-medusa_query_result_get_next_record (MedusaQueryResult *result)
-{
-	return result->next_record;
-}
-
-void
-medusa_query_result_set_succeeded (MedusaQueryResult *result,
-				   gboolean value)
-{
-	result->result = value;
-}
-
-void
-medusa_query_result_set_next_record (MedusaQueryResult *result,
-				     MedusaRDBRecordNumbers *next_record)
-{
-	result->next_record = next_record;
-}
-
-void
-medusa_query_result_free (MedusaQueryResult *result)
-{
-	g_free (result);
-}
 
 static void
 medusa_query_clauses_destroy (MedusaQueryClauses *clauses)
@@ -174,8 +232,8 @@ medusa_query_clauses_destroy (MedusaQueryClauses *clauses)
 }
 
 static MedusaQueryClause *     
-medusa_query_clause_new (char *field_name, 
-			 char *operator_name, 
+medusa_query_clause_new (const char *field_name, 
+			 const char *operator_name, 
 			 MedusaQueryFunc evaluate,
 			 MedusaArgumentType argument_type)
 {
@@ -191,6 +249,24 @@ medusa_query_clause_new (char *field_name,
 	return clause;
 }
 
+static MedusaQueryClause *
+medusa_query_clause_new_with_check (const char *field_name,
+				    const char *operator_name,
+				    MedusaQueryFunc evaluate,
+				    MedusaResultCheckFunc verify,
+				    MedusaArgumentType argument_type)
+{
+	MedusaQueryClause *clause;
+
+	clause = medusa_query_clause_new (field_name,
+					  operator_name,
+					  evaluate,
+					  argument_type);
+	clause->verify = verify;
+
+	return clause;
+}
+
 /*
   static void
   medusa_query_clause_ref (MedusaQueryClause *clause)
