@@ -27,10 +27,13 @@
  */
 
 #include <glib.h>
+#include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 #include <time.h>
 
+#include <libmedusa/medusa-index-service.h>
+#include <libmedusa/medusa-string.h>
 #include <libmedusa/medusa-utils.h>
 #include <libmedusa/medusa-file-info-utilities.h>
 
@@ -68,7 +71,7 @@ static void                         query_optimization_free              (gpoint
                                                                           gpointer user_data);
 static gint                         optimization_is_applicable           (gconstpointer data,
                                                                           gconstpointer user_data);
-static gboolean                     result_is_constant_query             (MedusaOptimizationResult *result);
+static gboolean                     result_is_error                      (MedusaOptimizationResult *result);
 
 static MedusaOptimizationResult *   optimization_result_new              (void);
 
@@ -81,6 +84,13 @@ static gboolean                     owner_query_relation_is_invalid      (const
 static gboolean                     can_replace_username_with_uid        (MedusaUnparsedCriteria *criteria);
 static MedusaOptimizationResult *   replace_username_with_uid_criterion  (MedusaUnparsedCriteria *criteria);
 
+static gboolean                     request_is_too_new_for_database      (MedusaUnparsedCriteria *criteria);
+static MedusaOptimizationResult *   set_error_request_obsoletes_index    (MedusaUnparsedCriteria *criteria);
+
+static gboolean                     request_is_newer_than_the_present    (MedusaUnparsedCriteria *criteria);
+static MedusaOptimizationResult *   set_error_always_false               (MedusaUnparsedCriteria *criteria);
+
+
 static const char *                 group_relation_to_corresponding_gid_relation (const char *query_relation);
 static gboolean                     group_query_relation_is_invalid      (const char *query_relation);
 static gboolean                     can_replace_group_with_gid           (MedusaUnparsedCriteria *criteria);
@@ -124,6 +134,14 @@ medusa_query_optimizations_initialize  (void)
                                         can_replace_modified_with_mtime,
                                         replace_modified_with_mtime_criteria,
                                         MEDUSA_OPTIMIZATION_RUN_REGULAR);
+        medusa_query_optimizations_add (optimizations,
+                                        request_is_too_new_for_database,
+                                        set_error_request_obsoletes_index,
+                                        MEDUSA_OPTIMIZATION_RUN_LAST);
+        medusa_query_optimizations_add (optimizations,
+                                        request_is_newer_than_the_present,
+                                        set_error_always_false,
+                                        MEDUSA_OPTIMIZATION_RUN_LAST);
         return optimizations;
 }
 
@@ -146,7 +164,7 @@ medusa_query_optimizations_perform_and_free_deep (MedusaQueryOptimizationList *o
                                                         optimization_is_applicable)) != NULL) {
                 relevant_optimization = optimization_list->data;
                 result = relevant_optimization->optimize (result->criteria);
-                if (result_is_constant_query (result)) {
+                if (result_is_error (result)) {
                         return result;
                 }
         }
@@ -156,7 +174,7 @@ medusa_query_optimizations_perform_and_free_deep (MedusaQueryOptimizationList *o
                                                         optimization_is_applicable)) != NULL) {
                 relevant_optimization = optimization_list->data;
                 result = relevant_optimization->optimize (result->criteria);
-                if (result_is_constant_query (result)) {
+                if (result_is_error (result)) {
                         return result;
                 }
         }
@@ -166,7 +184,7 @@ medusa_query_optimizations_perform_and_free_deep (MedusaQueryOptimizationList *o
                                                         optimization_is_applicable)) != NULL) {
                 relevant_optimization = optimization_list->data;
                 result = relevant_optimization->optimize (result->criteria);
-                if (result_is_constant_query (result)) {
+                if (result_is_error (result)) {
                         return result;
                 }
         }
@@ -191,6 +209,9 @@ medusa_query_optimizations_new (void)
 void
 medusa_query_optimizations_destroy (MedusaQueryOptimizationList *optimizations)
 {
+        if (optimizations == NULL) {
+                return;
+        }
         if (optimizations->run_first_optimizations != NULL) {
                 medusa_g_list_free_deep_custom (optimizations->run_first_optimizations,
                                                 query_optimization_free,
@@ -268,10 +289,13 @@ optimization_is_applicable (gconstpointer data,
         MedusaUnparsedCriteria *criteria;
 
         g_assert (data != NULL);
-        g_assert (user_data != NULL);
         optimization = (MedusaQueryOptimization *) data;
         criteria = (MedusaUnparsedCriteria *) user_data;
 
+        /* You can't optimize nothing */
+        if (criteria == NULL) {
+                return 1;
+        }
         if (optimization->optimization_is_applicable (criteria)) {
                 return 0;
         }
@@ -281,9 +305,9 @@ optimization_is_applicable (gconstpointer data,
 }
 
 static gboolean                 
-result_is_constant_query (MedusaOptimizationResult *result)
+result_is_error (MedusaOptimizationResult *result)
 {
-        return result->query_is_always_true || result->query_is_always_false;
+        return result->error != MEDUSA_SEARCH_URI_NO_ERROR;
 }
 
 
@@ -293,9 +317,7 @@ optimization_result_new (void)
         MedusaOptimizationResult *optimization_result;
 
         optimization_result = g_new0 (MedusaOptimizationResult, 1);
-        optimization_result->syntax_error_found = FALSE;
-        optimization_result->query_is_always_true = FALSE;
-        optimization_result->query_is_always_false = FALSE;
+        optimization_result->error = MEDUSA_SEARCH_URI_NO_ERROR;
         optimization_result->criteria = NULL;
 
         return optimization_result;
@@ -424,6 +446,73 @@ can_replace_username_with_uid (MedusaUnparsedCriteria *criteria)
         return FALSE;
 }
 
+static gboolean                     
+request_is_too_new_for_database (MedusaUnparsedCriteria *criteria)
+{
+        int i;
+        time_t after_time, time_index_was_made;
+        const char *time_as_string;
+
+        for (i = 0; criteria[i] != NULL; i++) {
+                if (medusa_str_has_prefix (criteria[i], "mtime is_after")) {
+                        time_as_string = criteria[i] + strlen ("mtime is_after");
+                        after_time = strtol (time_as_string, NULL, 10);
+                        time_index_was_made = medusa_index_service_get_last_index_update_time ();
+                        if (after_time > time_index_was_made  &&
+                            after_time <= time (NULL)) {
+                                return TRUE;
+                        }
+                }
+        }
+
+        return FALSE;
+}
+
+static MedusaOptimizationResult *   
+set_error_request_obsoletes_index (MedusaUnparsedCriteria *criteria)
+{
+        MedusaOptimizationResult *result;
+
+        result = optimization_result_new ();
+        g_strfreev (result->criteria);
+        result->criteria = NULL;
+        result->error = MEDUSA_SEARCH_URI_OBSOLETES_INDEX;
+        return result;
+}
+
+
+static gboolean                     
+request_is_newer_than_the_present (MedusaUnparsedCriteria *criteria)
+{
+        int i;
+        time_t after_time;
+        const char *time_as_string;
+
+        for (i = 0; criteria[i] != NULL; i++) {
+                if (medusa_str_has_prefix (criteria[i], "mtime is_after")) {
+                        time_as_string = criteria[i] + strlen ("mtime is_after");
+                        after_time = strtol (time_as_string, NULL, 10);
+                        if (after_time > time (NULL)) {
+                                return TRUE;
+                        }
+                }
+        }
+
+        return FALSE;
+}
+
+static MedusaOptimizationResult *   
+set_error_always_false (MedusaUnparsedCriteria *criteria)
+{
+        MedusaOptimizationResult *result;
+
+        result = optimization_result_new ();
+        g_strfreev (result->criteria);
+        result->criteria = NULL;
+        result->error = MEDUSA_SEARCH_URI_IS_ALWAYS_FALSE;
+        return result;
+}
+
 
 static gboolean
 owner_query_relation_is_invalid (const char *query_relation)
@@ -468,13 +557,13 @@ replace_username_with_uid_criterion (MedusaUnparsedCriteria *criteria)
                                                        "owner is");
         query_relation = query_criterion_get_relation (old_criterion);
         if (query_relation == NULL) {
-                result->syntax_error_found = TRUE;
+                result->error = MEDUSA_SEARCH_URI_SYNTAX_ERROR;
                 return result;
         }
         query_object = query_criterion_get_object (old_criterion);
         if (owner_query_relation_is_invalid (query_relation) ||
             query_object == NULL) {
-                result->syntax_error_found = TRUE;
+                result->error = MEDUSA_SEARCH_URI_SYNTAX_ERROR;
                 return result;
         }
         if (medusa_username_to_uid (query_object,
@@ -490,10 +579,10 @@ replace_username_with_uid_criterion (MedusaUnparsedCriteria *criteria)
         }
         else {
                 if (query_relation_is_negative (query_relation)) {
-                        result->query_is_always_true = TRUE;
+                        result->error = MEDUSA_SEARCH_URI_IS_ALWAYS_TRUE;
                 }
                 else {
-                        result->query_is_always_false = TRUE;
+                        result->error = MEDUSA_SEARCH_URI_IS_ALWAYS_FALSE;
                 }
         }
         
@@ -561,7 +650,7 @@ replace_group_with_gid_criterion (MedusaUnparsedCriteria *criteria)
         query_object = query_criterion_get_object (old_criterion);
         if (group_query_relation_is_invalid (query_relation) ||
             query_object == NULL) {
-                result->syntax_error_found = TRUE;
+                result->error = MEDUSA_SEARCH_URI_SYNTAX_ERROR;
                 return result;
         }
         if (medusa_group_to_gid (query_object,
@@ -577,10 +666,10 @@ replace_group_with_gid_criterion (MedusaUnparsedCriteria *criteria)
         }
         else {
                 if (query_relation_is_negative (query_relation)) {
-                        result->query_is_always_true = TRUE;
+                        result->error = MEDUSA_SEARCH_URI_IS_ALWAYS_TRUE;
                 }
                 else {
-                        result->query_is_always_false = TRUE;
+                        result->error = MEDUSA_SEARCH_URI_IS_ALWAYS_FALSE;
                 }
         }
         
@@ -686,13 +775,8 @@ last_time_corresponding_to_query_object (const char *query_relation,
         time_t relevant_time;
 
          date = object_to_date (object);
+         
          if (medusa_str_has_prefix (query_relation,
-                                    "is") ||
-             medusa_str_has_prefix (query_relation,
-                                    "is_not")) {
-                 relevant_time = medusa_file_info_get_last_unix_time_occurring_on_date (date);
-         }
-         else if (medusa_str_has_prefix (query_relation,
                                          "is_before")) {
                  relevant_time = medusa_file_info_get_first_unix_time_occurring_on_date (date);
          }
@@ -704,9 +788,15 @@ last_time_corresponding_to_query_object (const char *query_relation,
                                          "is_within_a_month_of")) {
                  relevant_time = medusa_file_info_get_unix_time_a_month_after_date (date);
          }                          
+         else if (medusa_str_has_prefix (query_relation,
+                                         "is") ||
+                  medusa_str_has_prefix (query_relation,
+                                         "is_not")) {
+                 relevant_time = medusa_file_info_get_last_unix_time_occurring_on_date (date);
+         }
+ 
          else {
-                 g_assert_not_reached ();
-                 relevant_time = 0;
+                  relevant_time = 0;
          }
          g_free (date);
          return relevant_time;
@@ -741,7 +831,7 @@ replace_modified_with_mtime_criteria (MedusaUnparsedCriteria *criteria)
          query_object = query_criterion_get_object (old_criterion);
          if (modified_query_relation_is_invalid (query_relation) ||
              query_object == NULL) {
-                 result->syntax_error_found = TRUE;
+                 result->error = MEDUSA_SEARCH_URI_SYNTAX_ERROR;
                  return result;
          }
 
