@@ -26,14 +26,16 @@
 
 #include <glib.h>
 #include <string.h>
+#include <time.h>
 #include <unistd.h>
 
 #include <libgnomevfs/gnome-vfs-types.h>
 #include <libgnomevfs/gnome-vfs-directory.h>
-
+#include <libmedusa/medusa-log.h>
+#include <medusa-idled/medusa-idled-client.h>
 #include "medusa-conf.h"
-#include "medusa-enums.h"
 #include "medusa-file-index.h"
+#include "medusa-index-filenames.h"
 #include "medusa-io-handler.h"
 #include "medusa-hash.h"
 #include "medusa-text-index.h"
@@ -42,6 +44,12 @@
 #include "medusa-text-index-plaintext-module.h"
 
 
+#define START_FILE_PREFIX  "text-index-start-file"
+#define LOCATION_FILE_PREFIX "text-index-location-file"
+#define WORD_FILE_PREFIX  "text-index-word-file"
+#define TEMP_FILE_PREFIX  "text-index-temp-file"
+
+#define WORD_HASH_BITS 21
 
 #define MEDUSA_TEXT_INDEX_TEMP_FILE_MAGIC_NUMBER         "9124"
 #define MEDUSA_TEXT_INDEX_TEMP_FILE_VERSION_NUMBER       "0.1"
@@ -50,6 +58,8 @@
 #define MEDUSA_TEXT_INDEX_LOCATIONS_INDEX_MAGIC_NUMBER   "9126"
 #define MEDUSA_TEXT_INDEX_LOCATIONS_INDEX_VERSION_NUMBER "0.1"
 
+/* FIXME: These macros should be replaced by the new log functions 
+   in medusa-log */
 #define LOG_OUTPUT(args) \
 if (text_index->log_level == MEDUSA_DB_LOG_EVERYTHING) { \
         printf args; \
@@ -67,26 +77,19 @@ if (text_index->log_level == MEDUSA_DB_LOG_TEXT_INDEX_DATA) { \
 }
 
 
-/* Before creating a text index we want to be sure that we aren't overwriting 
-   anything when we make new files */
-static gboolean             permenant_text_index_files_exist          (const char *start_index_file,
-                                                                       const char *locations_index_file,
-                                                                       const char *semantic_units_index_file);
-static gboolean             text_index_files_dont_exist               (const char *start_index_file,
-                                                                       const char *locations_index_file,
-                                                                       const char *semantic_units_index_file,
-                                                                       const char *temp_index_file);
 /* ie text index structure is set up, and all files were
    found or made successfully */
 static gboolean             text_index_files_are_ready                (MedusaTextIndex *text_index);
-static char *               make_temp_index_file_name                 (const char *temp_index_file, 
+static char *               generate_temp_index_file_name             (const char *index_name, 
                                                                        int file_number);
 static void                 array_of_words_free                       (char **array, 
                                                                        int number_of_words);
 static void                 text_index_add_mime_modules               (MedusaTextIndex *text_index);
 
 static void                 setup_temp_index_io_handlers              (MedusaTextIndex *text_index);
-static void                 sort_temp_index_data_into_permenant_index (MedusaTextIndex *text_index,
+static void                 sort_temp_index_data_into_permanent_index (MedusaTextIndex *text_index,
+                                                                       gboolean use_idle_service,
+                                                                       MedusaIdledConnection *idled_connection,
                                                                        int temp_index_number);
 static void                 add_word_to_real_index                    (gpointer key,
                                                                        gpointer value,
@@ -112,33 +115,48 @@ static void                 write_uri_number_to_location_file     (MedusaTextInd
 static void                 medusa_text_index_destroy             (MedusaTextIndex *text_index);
 
 MedusaTextIndex *
-medusa_text_index_open (const char *start_index_file,
-                        MedusaLogLevel log_level,
-                        const char *locations_index_file,
-                        const char *semantic_units_index_file)
+medusa_text_index_open (const char *index_name,
+                        MedusaLogLevel log_level)
 {
         MedusaTextIndex *text_index;
         MedusaVersionedFileResult start_index_open_result, locations_index_open_result;
-
-        g_return_val_if_fail (permenant_text_index_files_exist (start_index_file,
-                                                                locations_index_file,
-                                                                semantic_units_index_file), NULL);
+        char *start_index_filename, *locations_index_filename, *word_index_filename;
 
         text_index = g_new0 (MedusaTextIndex, 1);
 
         text_index->creating_index = FALSE;
-        text_index->semantic_units = medusa_hash_open (semantic_units_index_file, 
-                                                       SEMANTIC_UNITS_BITS);
-        text_index->start_index_name = g_strdup (start_index_file);
-        text_index->start_index = medusa_versioned_file_open (start_index_file,
+
+
+        word_index_filename = medusa_generate_index_filename (WORD_FILE_PREFIX,
+                                                              index_name,
+                                                              FALSE);
+        text_index->semantic_units = medusa_hash_open (word_index_filename,
+                                                       WORD_HASH_BITS);
+        g_free (word_index_filename);
+        if (text_index->semantic_units == NULL) {
+                g_warning ("Couldn't create word hash file\n");
+                /* FIXME bugzilla.eazel.com 4557: log errors here */
+                g_free (text_index);
+                return NULL;
+        }
+
+        start_index_filename = medusa_generate_index_filename (START_FILE_PREFIX,
+                                                               index_name,
+                                                               FALSE);
+        text_index->start_index = medusa_versioned_file_open (start_index_filename,
                                                               MEDUSA_TEXT_INDEX_START_INDEX_MAGIC_NUMBER,
                                                               MEDUSA_TEXT_INDEX_START_INDEX_VERSION_NUMBER,
                                                               &start_index_open_result);
-        text_index->locations_index_name = g_strdup (locations_index_file);
-        text_index->locations_index = medusa_versioned_file_open (locations_index_file,
+
+        locations_index_filename = medusa_generate_index_filename (LOCATION_FILE_PREFIX,
+                                                                   index_name,
+                                                                   FALSE);
+        text_index->locations_index = medusa_versioned_file_open (locations_index_filename,
                                                                   MEDUSA_TEXT_INDEX_LOCATIONS_INDEX_MAGIC_NUMBER,
                                                                   MEDUSA_TEXT_INDEX_LOCATIONS_INDEX_VERSION_NUMBER,
                                                                   &locations_index_open_result);
+        g_free (locations_index_filename);
+
         text_index->log_level = log_level;
         text_index->ref_count = 1;
 
@@ -152,31 +170,39 @@ medusa_text_index_open (const char *start_index_file,
                 medusa_text_index_unref (text_index);
                 return NULL;
         }
+
         return text_index;
 
 }
 
 MedusaTextIndex *
-medusa_text_index_new (const char *start_index_file,
-                       MedusaLogLevel log_level,
-                       const char *locations_index_file,
-                       const char *semantic_units_index_file,
-                       const char *temp_index_file)
+medusa_text_index_new (const char *index_name,
+                       MedusaLogLevel log_level)
 {
         MedusaTextIndex *text_index;
         int zero, i;
         MedusaVersionedFileResult start_index_create_result, locations_index_create_result;
+        char *start_file_name, *location_file_name, *word_file_name;
+        char *temp_file_root_name;
         
-        g_return_val_if_fail (text_index_files_dont_exist (start_index_file,
-                                                           locations_index_file,
-                                                           semantic_units_index_file,
-                                                           temp_index_file), NULL);
         
+        start_file_name = medusa_generate_index_filename (START_FILE_PREFIX,
+                                                          index_name,
+                                                          TRUE);
+        location_file_name = medusa_generate_index_filename (LOCATION_FILE_PREFIX,
+                                                             index_name,
+                                                             TRUE);
+        word_file_name = medusa_generate_index_filename (WORD_FILE_PREFIX,
+                                                         index_name,
+                                                         TRUE);
+        temp_file_root_name = medusa_generate_index_filename (TEMP_FILE_PREFIX,
+                                                              index_name,
+                                                              TRUE);
         text_index = g_new0 (MedusaTextIndex, 1);
         text_index->creating_index = TRUE;
 
-        text_index->semantic_units = medusa_hash_new (semantic_units_index_file, 
-                                                      SEMANTIC_UNITS_BITS);
+        text_index->semantic_units = medusa_hash_new (word_file_name,
+                                                      WORD_HASH_BITS);
         text_index->last_occurrence = g_hash_table_new (g_direct_hash,
                                                         g_direct_equal);
 
@@ -188,7 +214,7 @@ medusa_text_index_new (const char *start_index_file,
         text_index->temp_indices_are_memory_mapped = FALSE;
 
         for (i = 0; i < NUMBER_OF_TEMP_INDEXES; i++) {
-                text_index->temp_index_name[i] = make_temp_index_file_name (temp_index_file, i);
+                text_index->temp_index_name[i] = generate_temp_index_file_name (index_name, i);
                 text_index->temp_index_stream[i] = 
                         fopen_new_with_medusa_io_handler_header (text_index->temp_index_name[i],
                                                                  MEDUSA_TEXT_INDEX_TEMP_FILE_MAGIC_NUMBER,
@@ -206,8 +232,7 @@ medusa_text_index_new (const char *start_index_file,
                         text_index->current_cell_number[i] = 1;
         }
 
-        text_index->start_index_name = g_strdup (start_index_file);
-        text_index->start_index = medusa_versioned_file_create (start_index_file,
+        text_index->start_index = medusa_versioned_file_create (start_file_name,
                                                                 MEDUSA_TEXT_INDEX_START_INDEX_MAGIC_NUMBER,
                                                                 MEDUSA_TEXT_INDEX_START_INDEX_VERSION_NUMBER,
                                                                 &start_index_create_result);
@@ -217,8 +242,7 @@ medusa_text_index_new (const char *start_index_file,
         text_index->reverse_index_position = 1;
         
 
-        text_index->locations_index_name = g_strdup (locations_index_file);
-        text_index->locations_index = medusa_versioned_file_create (locations_index_file,
+        text_index->locations_index = medusa_versioned_file_create (location_file_name,
                                                                     MEDUSA_TEXT_INDEX_LOCATIONS_INDEX_MAGIC_NUMBER,
                                                                     MEDUSA_TEXT_INDEX_LOCATIONS_INDEX_VERSION_NUMBER,
                                                                     &locations_index_create_result);
@@ -226,6 +250,11 @@ medusa_text_index_new (const char *start_index_file,
                 ABBREV_LOG_OUTPUT (("Could not create locations index file with error code %d\n", locations_index_create_result));
         }
 
+        g_free (start_file_name);
+        g_free (location_file_name);
+        g_free (word_file_name);
+        g_free (temp_file_root_name);
+
         medusa_versioned_file_write (text_index->locations_index, &zero, sizeof (gint32), 1);
         text_index_add_mime_modules (text_index);
 
@@ -281,53 +310,23 @@ get_last_cell_from_temp_index_cell (MedusaTextIndex *text_index,
 }
 
 static char *
-make_temp_index_file_name (const char *temp_index_file, 
-                           int file_number)
+generate_temp_index_file_prefix (int file_number)
 {
-        return g_strdup_printf ("%s.%d", temp_index_file, file_number);
+        return g_strdup_printf ("%s-%d", TEMP_FILE_PREFIX, file_number);
 }
 
-static gboolean
-permenant_text_index_files_exist (const char *start_index_file,
-                                  const char *locations_index_file,
-                                  const char *semantic_units_index_file)
-{
-       if (access (start_index_file, X_OK) == -1) {
-                return FALSE;
-        }
-        if (access (locations_index_file, X_OK) == -1) {
-                return FALSE;
-        }
-        if (access (semantic_units_index_file, X_OK) == -1) {
-                return FALSE;
-        }
- 
-        return TRUE;
-}
+static char *
+generate_temp_index_file_name (const char *index_name,
+                               int file_number)
+{ 
+        char *temp_file_prefix;
+        char *temp_filename;
+                
+        temp_file_prefix = generate_temp_index_file_prefix (file_number);
+        temp_filename = medusa_generate_index_filename (temp_file_prefix, index_name, TRUE);
+        g_free (temp_file_prefix);
 
-static gboolean             
-text_index_files_dont_exist (const char *start_index_file,
-                             const char *locations_index_file,
-                             const char *semantic_units_index_file,
-                             const char *temp_index_file)
-{
-        int i;
-        char *one_temp_index_name;
-
-        if (permenant_text_index_files_exist (start_index_file,
-                                              locations_index_file,
-                                              semantic_units_index_file)) {
-                return FALSE;
-        }
-        for (i = 0; i < NUMBER_OF_TEMP_INDEXES; i++) {
-                one_temp_index_name = make_temp_index_file_name (temp_index_file, i); 
-                if (access (one_temp_index_name, X_OK) != -1) {
-                        return FALSE;
-                }
-                g_free (one_temp_index_name);
-        }
-
-        return TRUE;
+        return temp_filename;
 }
 
 static gboolean
@@ -406,7 +405,9 @@ medusa_text_index_read_file (MedusaTextIndex *text_index,
 }
 
 void
-medusa_text_index_finish_indexing (MedusaTextIndex *text_index)
+medusa_text_index_finish_indexing (MedusaTextIndex *text_index,
+                                   gboolean use_idle_service,
+                                   MedusaIdledConnection *idled_connection)
 {
         unsigned int i;
         
@@ -422,10 +423,69 @@ medusa_text_index_finish_indexing (MedusaTextIndex *text_index)
         for (i = 0; i < NUMBER_OF_TEMP_INDEXES; i++) {
                 /* We can filter each small index, serially,
                    which cuts down greatly on memory usage */
-                sort_temp_index_data_into_permenant_index (text_index, i);
+                sort_temp_index_data_into_permanent_index (text_index, use_idle_service, idled_connection, i);
         }
 }
 
+void                     
+medusa_text_index_erase_constructed_index (const char *index_name)
+{
+        char *current_temp_file_prefix;
+        int i;
+
+        medusa_erase_constructed_index_file (START_FILE_PREFIX,
+                                             index_name);
+        medusa_erase_constructed_index_file (LOCATION_FILE_PREFIX,
+                                             index_name);
+        medusa_erase_constructed_index_file (WORD_FILE_PREFIX,
+                                             index_name);
+        for (i = 0; i < NUMBER_OF_TEMP_INDEXES; i++) {
+                current_temp_file_prefix = generate_temp_index_file_prefix (i);
+                medusa_erase_constructed_index_file (current_temp_file_prefix,
+                                                     index_name);
+                g_free (current_temp_file_prefix);
+        }
+        
+}
+
+void                     
+medusa_text_index_move_completed_index_into_place (const char *index_name)
+{
+        char *current_temp_file_prefix;
+        int i;
+
+        medusa_move_completed_index_file_into_place (START_FILE_PREFIX,
+                                                     index_name);
+        medusa_move_completed_index_file_into_place (LOCATION_FILE_PREFIX,
+                                                     index_name);
+        medusa_move_completed_index_file_into_place (WORD_FILE_PREFIX,
+                                                     index_name);
+        for (i = 0; i < NUMBER_OF_TEMP_INDEXES; i++) {
+                current_temp_file_prefix = generate_temp_index_file_prefix (i);
+                medusa_erase_constructed_index_file (current_temp_file_prefix,
+                                                     index_name);
+                g_free (current_temp_file_prefix);
+        }
+        
+}
+
+gboolean                 
+medusa_text_index_files_are_still_valid (const char *index_name,
+                                         time_t newest_valid_modified_time)
+{
+        return medusa_index_file_is_newer_than_time (START_FILE_PREFIX,
+                                                     index_name,
+                                                     newest_valid_modified_time) &&
+                medusa_index_file_is_newer_than_time (LOCATION_FILE_PREFIX,
+                                                      index_name,
+                                                      newest_valid_modified_time) &&
+                medusa_index_file_is_newer_than_time (WORD_FILE_PREFIX,
+                                                      index_name,
+                                                      newest_valid_modified_time);
+
+}
+
+
 
 static void
 setup_temp_index_io_handlers (MedusaTextIndex *index)
@@ -448,12 +508,32 @@ setup_temp_index_io_handlers (MedusaTextIndex *index)
         }
 }
 
+/* Interval is in seconds */
+#define IDLE_CHECK_TIME_INTERVAL 2
+
+static void
+sleep_until_idle_when_idle_check_interval_has_passed (MedusaIdledConnection *idled_connection,
+                                                      time_t *time_of_last_idle_check)
+{
+        time_t current_time;
+        current_time = time (NULL);
+        if ((current_time - *time_of_last_idle_check) > IDLE_CHECK_TIME_INTERVAL) {
+                medusa_idle_service_sleep_until_idle (idled_connection);
+                *time_of_last_idle_check = time (NULL);
+        }
+
+        return;
+}
+
 static void                 
-sort_temp_index_data_into_permenant_index (MedusaTextIndex *index,
+sort_temp_index_data_into_permanent_index (MedusaTextIndex *index,
+                                           gboolean use_idled_service,
+                                           MedusaIdledConnection *idled_connection,
                                            int temp_index_number)
 {
         int j;
         char *next_word;
+        time_t time_of_last_idle_check;
         
         for (j = temp_index_number; j < (1 << index->semantic_units->key_bits); j += NUMBER_OF_TEMP_INDEXES) {
                 next_word = medusa_token_to_string (index->semantic_units, j);
@@ -465,6 +545,11 @@ sort_temp_index_data_into_permenant_index (MedusaTextIndex *index,
                                                 g_hash_table_lookup (index->last_occurrence,
                                                                      GINT_TO_POINTER (j)),
                                                 index);
+                        if (use_idled_service) {
+                                sleep_until_idle_when_idle_check_interval_has_passed (idled_connection,
+                                                                                       &time_of_last_idle_check);
+                                medusa_idle_service_sleep_until_idle (idled_connection);
+                        }
                 }
                 
         }
@@ -513,7 +598,7 @@ add_word_to_real_index (gpointer key,
         
 }
 
-/* FIXME: These three functions should return codes in case of error */        
+/* FIXME bugzilla.eazel.com 4563: These three functions should return codes in case of error */        
 static void                 
 write_start_location_to_start_file (MedusaTextIndex *text_index,
                                        MedusaToken word_token,
@@ -706,10 +791,8 @@ medusa_text_index_destroy (MedusaTextIndex *text_index)
                 g_list_free (text_index->mime_modules);
         }
 
-        g_free (text_index->start_index_name);
         medusa_versioned_file_destroy (text_index->start_index);
         
-        g_free (text_index->locations_index_name);
         medusa_versioned_file_destroy (text_index->locations_index);
 
         g_free (text_index);
