@@ -38,15 +38,20 @@
 
 #include "medusa-conf.h"
 #include "medusa-file-index.h"
-#include "medusa-hash.h"
+#include "medusa-index-filenames.h"
 #include "medusa-rdb-table.h"
+#include "medusa-string-list.h"
 #include "medusa-uri-list.h"
 #include "medusa-uri-list-encoders.h"
 #include "medusa-uri-list-private.h"
 #include "medusa-uri-list-queries.h"
 
 
+#define URI_LIST_NAME "uri-list"
 #define URI_LIST_METAINFO_LENGTH 100
+#define FILE_NAME_LIST_PREFIX "file-names"
+#define DIRECTORY_LIST_PREFIX "directory-names"
+
 
 static MedusaRDBFieldInfo * uri_list_field_info               (void);
 static MedusaQueryClauses * uri_list_clauses                  (void);
@@ -62,18 +67,47 @@ static void                 medusa_uri_list_destroy           (MedusaURIList *db
 
 
 MedusaURIList *
-medusa_uri_list_new (const char *root_directory,
-		     MedusaLogLevel log_level,
-		     const char *uri_list_name,
-		     MedusaHash *file_names,
-		     MedusaHash *directory_names) 
+medusa_uri_list_new (const char *index_name,
+		     MedusaLogLevel log_level)
 {
 	MedusaURIList *uri_list;
 	MedusaRDBFile *uri_file;
+	char *file_list_name, *directory_list_name;
+	MedusaStringList *file_names, *directory_names;
+	char *uri_list_name;
+	file_list_name = medusa_generate_index_filename (FILE_NAME_LIST_PREFIX, 
+							 index_name,
+							 TRUE);
+	unlink (file_list_name);
+	file_names = medusa_string_list_new (file_list_name,
+					     MEDUSA_STRING_LIST_NO_WRITE_CACHE);
+	if (file_names == NULL) {
+		medusa_log_event ("Error creating file name list\n", MEDUSA_DB_LOG_ERRORS);
+		return NULL;
+	}
+	g_free (file_list_name);
+
+	directory_list_name = medusa_generate_index_filename (DIRECTORY_LIST_PREFIX,
+							      index_name,
+							      TRUE);
+	
+	unlink (directory_list_name);
+	directory_names = medusa_string_list_new (directory_list_name,
+						  MEDUSA_STRING_LIST_SINGLE_ELEMENT_WRITE_CACHE);
+	g_free (directory_list_name);
+	if (file_names == NULL) {
+		medusa_log_event ("Error creating directory name list\n", MEDUSA_DB_LOG_ERRORS);
+		medusa_string_list_destroy (file_names);
+		return NULL;
+	}
 
-	g_return_val_if_fail (access (uri_list_name, X_OK) == -1, NULL);
 	uri_list = g_new0 (MedusaURIList, 1);
+	uri_list_name = medusa_generate_index_filename (URI_LIST_NAME,
+							index_name,
+							TRUE);
 	uri_file = medusa_rdb_file_new (uri_list_name, URI_LIST_METAINFO_LENGTH);
+	g_free (uri_list_name);
+
 
 	medusa_rdb_file_add_field (uri_file,
 				   MEDUSA_URI_LIST_DIRECTORY_NAME_FIELD_TITLE,
@@ -95,7 +129,6 @@ medusa_uri_list_new (const char *root_directory,
 
 	uri_list->file_names = file_names;
 	uri_list->directory_names = directory_names;
-	uri_list->root_directory = g_strdup (root_directory);
 	uri_list->clauses = uri_list_clauses ();
 	uri_list->log_level = log_level;
 	uri_list->ref_count = 1;
@@ -106,20 +139,54 @@ medusa_uri_list_new (const char *root_directory,
 }
 
 MedusaURIList *
-medusa_uri_list_open (const char *root_directory,
-		     MedusaLogLevel log_level,
-		     const char *uri_list_name,
-		     MedusaHash *file_names,
-		     MedusaHash *directory_names) 
+medusa_uri_list_open (const char *index_name,
+		      MedusaLogLevel log_level)
 {
 	MedusaURIList *uri_list;
 	MedusaRDBFile *uri_file;
+	MedusaStringList *file_names = NULL, *directory_names = NULL;
+	char *file_list_name, *directory_list_name;
+	char *uri_list_file_name;
 
-	g_return_val_if_fail (access (uri_list_name, R_OK) != -1, NULL);
+	uri_list_file_name = medusa_generate_index_filename (URI_LIST_NAME,
+							     index_name,
+							     FALSE);
+	uri_file = medusa_rdb_file_open (uri_list_file_name, URI_LIST_METAINFO_LENGTH);
+	g_free (uri_list_file_name);
+
+	if (uri_file == NULL) {
+		medusa_log_event ("Could not open the URI database",
+				  MEDUSA_DB_LOG_ERRORS);
+		return NULL;
+	}
+	file_list_name = medusa_generate_index_filename (FILE_NAME_LIST_PREFIX,
+							 index_name,
+							 FALSE);
+	file_names = medusa_string_list_open (file_list_name,
+					      MEDUSA_STRING_LIST_NO_READ_CACHE);
+	g_free (file_list_name);
+	if (file_names == NULL) {
+		medusa_log_event ("Could not open the file name list", MEDUSA_DB_LOG_ERRORS);
+		medusa_rdb_file_free (uri_file);
+		return NULL;
+	}
+	
+	directory_list_name = medusa_generate_index_filename (DIRECTORY_LIST_PREFIX,
+							      index_name,
+							      FALSE);
+	directory_names = medusa_string_list_open (directory_list_name,
+					    MEDUSA_STRING_LIST_NO_READ_CACHE);
+	g_free (directory_list_name);
+	if (directory_names == NULL) {
+
+		medusa_log_event ("Error opening directory list",
+				  MEDUSA_DB_LOG_ERRORS);
+		medusa_string_list_destroy (file_names);
+		medusa_rdb_file_free (uri_file);
+		return NULL;
+	}
 
 	uri_list = g_new0 (MedusaURIList, 1);
-	uri_file = medusa_rdb_file_open (uri_list_name, URI_LIST_METAINFO_LENGTH);
-
 	uri_file->field_info = uri_list_field_info ();
 	uri_list->uri_names = medusa_rdb_table_all_rows (uri_file);
 	read_index_info_from_database (uri_list);
@@ -128,7 +195,6 @@ medusa_uri_list_open (const char *root_directory,
 
 	uri_list->file_names = file_names;
 	uri_list->directory_names = directory_names;
-	uri_list->root_directory = g_strdup (root_directory);
 	uri_list->clauses = uri_list_clauses ();
 	uri_list->log_level = log_level;
 	uri_list->ref_count = 1;
@@ -138,6 +204,45 @@ medusa_uri_list_open (const char *root_directory,
 	return uri_list;
 }	      
 
+void                    
+medusa_uri_list_erase_constructed_index (const char           *index_name)
+{
+
+	medusa_erase_constructed_index_file (URI_LIST_NAME,
+					     index_name);
+	medusa_erase_constructed_index_file (FILE_NAME_LIST_PREFIX,
+					     index_name);
+	medusa_erase_constructed_index_file (DIRECTORY_LIST_PREFIX,
+					     index_name);
+
+}
+
+void                    
+medusa_uri_list_move_completed_index_into_place (const char           *index_name)
+{
+	medusa_move_completed_index_file_into_place (FILE_NAME_LIST_PREFIX,
+						     index_name);
+	medusa_move_completed_index_file_into_place (DIRECTORY_LIST_PREFIX,
+						     index_name);
+	medusa_move_completed_index_file_into_place (URI_LIST_NAME,
+						     index_name);
+}
+
+gboolean                
+medusa_uri_list_index_files_are_still_valid (const char *index_name,
+					     time_t newest_valid_modified_time)
+{
+	return medusa_index_file_is_newer_than_time (FILE_NAME_LIST_PREFIX, 
+						     index_name,
+						     newest_valid_modified_time) &&
+		medusa_index_file_is_newer_than_time (DIRECTORY_LIST_PREFIX,
+						      index_name,
+						      newest_valid_modified_time) &&
+		medusa_index_file_is_newer_than_time (URI_LIST_NAME,
+						      index_name,
+						      newest_valid_modified_time);
+}
+
 
 
 char *
@@ -360,10 +465,9 @@ medusa_uri_list_destroy (MedusaURIList *uri_list)
 				      uri_list->indexing_start_time);
 	medusa_rdb_table_free (uri_list->uri_names);
 	medusa_query_clauses_unref (uri_list->clauses);
-	medusa_hash_unref (uri_list->file_names);
-	medusa_hash_unref (uri_list->directory_names);
+	medusa_string_list_destroy (uri_list->file_names);
+	medusa_string_list_destroy (uri_list->directory_names);
 	g_free (uri_list->version);
-	g_free (uri_list->root_directory);
 	g_free (uri_list);
   
 }
