@@ -32,16 +32,35 @@
 #ifndef MEDUSA_SEARCH_SERVICE_PRIVATE_H
 #define MEDUSA_SEARCH_SERVICE_PRIVATE_H
 
-#define MAX_LINE 512
+
 #define SEARCH_SOCKET_PATH  "/tmp/medusa-search-server"
-
-#define COOKIE_REQUEST "gimme cookie"
 #define COOKIE_PATH "/tmp/medusa-cookies"
-#define SEARCH_FILE_TRANSMISSION "File"
-#define SEARCH_END_TRANSMISSION "End"
-#define SEARCH_COOKIE_ACKNOWELEDGMENT_TRANSMISSION "Get your cookie\n"
-#define SEARCH_INDEX_ERROR_TRANSMISSION "Index Error\n"
 
+/* These are the strings that the client and server use to communicate back 
+   and forth about the status of the search.  There are three communications steps:
+   1.  Authentication via cookie
+   2.  Opening the search uri, and reporting errors during this process
+   3.  Reading the search results, and returning errors for this process */
+
+
+/* Cookie protocol information */
+#define COOKIE_REQUEST "gimme cookie"
+#define SEARCH_COOKIE_ACKNOWELEDGMENT_TRANSMISSION "Got your cookie"
+
+/* Search URI opened transmissions */
+#define SEARCH_URI_OK_TRANSMISSION "Search URI OK"
+#define SEARCH_URI_ALL_RESULTS_VALID_TRANSMISSION "Whole Database Matches"
+#define SEARCH_URI_NO_RESULTS_TRANSMISSION "Query returned no results"
+#define SEARCH_URI_INDEX_OBSOLETE_ERROR_TRANSMISSION "Search URI obsoletes index"
+#define SEARCH_URI_SYNTAX_ERROR_TRANSMISSION "Search URI syntax error"
+
+/* Search URI results transmissions */
+#define START_SEARCH_REQUEST "Start Search"
+#define SEARCH_INDEX_ERROR_TRANSMISSION "Index Error"
+#define SEARCH_END_TRANSMISSION "End"
+
+
+#define MAX_LINE 512
 #endif /* MEDUSA_SEARCH_SERVICE_PRIVATE_H */
 
 
