@@ -34,7 +34,7 @@
 #include <libmedusa/medusa-unindexed-search.h>
 #include <libmedusa/medusa-search-service-private.h>
 #include <libmedusa/medusa-service-private.h>
-#include <libmedusa/medusa-utils.h>
+#include <libmedusa/medusa-string.h>
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
@@ -45,41 +45,52 @@
 
 typedef enum {
         SEARCH_MODE_UNINDEXED,
-        SEARCH_MODE_INDEXED
+        SEARCH_MODE_INDEXED,
+        SEARCH_MODE_INDEXED_AND_UNINDEXED
 } SearchMode;
 
 struct MedusaSearchServiceConnection {
         char *uri;
         MedusaIndexedSearch *indexed_search;
         MedusaUnindexedSearch *unindexed_search;
-
         MedusaSearchMethod method;
         SearchMode mode;
+        gboolean finished_reading_indexed_results;
 };
 
-static gboolean               uri_has_valid_header          (const char *uri);
-
-static MedusaSearchMethod     gnome_vfs_uri_to_search_method          (GnomeVFSURI *uri);
-static MedusaSearchMethod     uri_to_search_method                    (const char *uri);
+static gboolean               uri_has_valid_header                    (const char *uri);
+static gboolean               vfs_uri_to_search_method                (GnomeVFSURI *uri,
+                                                                       MedusaSearchMethod *method);
+static gboolean               uri_to_search_method                    (const char *uri,
+                                                                       MedusaSearchMethod *method);
 
 static char *                 uri_to_string_remove_extra_slashes      (const GnomeVFSURI *uri);
 GnomeVFSResult
 medusa_search_service_connection_is_available_for_uri (GnomeVFSURI *uri)
 {
         MedusaSearchMethod method;
+        gboolean uri_looks_valid;
 
-        method = gnome_vfs_uri_to_search_method (uri);
-
-        /* FIXME bugzilla.eazel.com 3002:  
-           Should we claim searches are available 
-           for full searches if the index is not? */
+        uri_looks_valid = vfs_uri_to_search_method (uri,
+                                                    &method);
+        if (!uri_looks_valid) {
+                return GNOME_VFS_ERROR_INVALID_URI;
+        }
         if (method == MEDUSA_SEARCH_METHOD_INDEX_ONLY) {
                 return medusa_indexed_search_is_available ();
         }
-        g_assert ((method == MEDUSA_SEARCH_METHOD_UNINDEXED_ONLY) || 
-                  (method == MEDUSA_SEARCH_METHOD_FULL_SEARCH));
-        /* We can always do an unindexed search */
-        return GNOME_VFS_OK;
+        else if (method == MEDUSA_SEARCH_METHOD_UNINDEXED_ONLY) {
+                /* We can do an unindexed search only if there are no searches for content */
+                return medusa_unindexed_search_is_available_for_uri (uri->text);
+        }
+        
+        /* otherwise, a search is available if either indexed or unindexed search
+           is available */
+        if (medusa_indexed_search_is_available () == GNOME_VFS_OK) {
+                return GNOME_VFS_OK;
+        }
+        return medusa_unindexed_search_is_available_for_uri (uri->text);
+        
 }
 
 
@@ -88,7 +99,9 @@ medusa_search_service_connection_new (const char *uri,
                                       GnomeVFSResult *result)
 {
         MedusaSearchServiceConnection *connection;
-        
+        GnomeVFSResult indexed_search_result, unindexed_search_result;
+        gboolean uri_looks_valid;
+
         if (!uri_has_valid_header (uri)) {
                 *result = GNOME_VFS_ERROR_INVALID_URI;
                 return NULL;
@@ -96,29 +109,86 @@ medusa_search_service_connection_new (const char *uri,
 
         connection = g_new0 (MedusaSearchServiceConnection, 1);
         connection->uri = g_strdup (uri);
-        connection->method = uri_to_search_method (uri);
+        uri_looks_valid = uri_to_search_method (uri,
+                                                &connection->method);
+        if (!uri_looks_valid) {
+                g_free (connection->uri);
+                g_free (connection);
+                *result = GNOME_VFS_ERROR_INVALID_URI;
+                return NULL;
+        }
         if (connection->method == MEDUSA_SEARCH_METHOD_INDEX_ONLY) {
-                connection->indexed_search = medusa_indexed_search_new (result);
+                connection->indexed_search = medusa_indexed_search_new (result, uri);
                 connection->unindexed_search = NULL;
-                connection->mode = SEARCH_MODE_INDEXED;
+                if (*result == GNOME_VFS_OK) {
+                        connection->mode = SEARCH_MODE_INDEXED;
+                        return connection;
+                }
+                else {
+                        g_free (connection->uri);
+                        g_free (connection);
+                        return NULL;
+                }
+        }
+        if (connection->method == MEDUSA_SEARCH_METHOD_INDEX_IF_AVAILABLE) {
+                connection->indexed_search = medusa_indexed_search_new (result, uri);
+                if (*result != GNOME_VFS_OK) {
+                        connection->unindexed_search = medusa_unindexed_search_new (result, uri);
+                        if (*result == GNOME_VFS_OK) {
+                                connection->mode = SEARCH_MODE_UNINDEXED;
+                        }
+                        else {
+                                g_free (connection->uri);
+                                g_free (connection);
+                                return NULL;
+                        }
+                }
+                else {
+                        connection->unindexed_search = NULL;
+                        connection->mode = SEARCH_MODE_INDEXED;
+                }
                 return connection;
         }
         if (connection->method == MEDUSA_SEARCH_METHOD_FULL_SEARCH) {
-                connection->mode = SEARCH_MODE_INDEXED;
-                connection->indexed_search = medusa_indexed_search_new (result);
-                if (*result != GNOME_VFS_OK) {
-                        return connection;
-                }
-                connection->unindexed_search = medusa_unindexed_search_new (result, uri);
 
+                connection->indexed_search = medusa_indexed_search_new (&indexed_search_result, uri);
+                connection->unindexed_search = medusa_unindexed_search_new (&unindexed_search_result, uri);
+                if (indexed_search_result == GNOME_VFS_OK &&
+                    unindexed_search_result == GNOME_VFS_OK) {
+                        connection->mode = SEARCH_MODE_INDEXED_AND_UNINDEXED;
+                        *result = GNOME_VFS_OK;
+
+                }
+                else if (unindexed_search_result == GNOME_VFS_OK) {
+                        connection->mode = SEARCH_MODE_UNINDEXED;
+                        *result = GNOME_VFS_OK;
+                }
+                else if (indexed_search_result == GNOME_VFS_OK) {
+                        connection->mode = SEARCH_MODE_INDEXED;
+                        *result = GNOME_VFS_OK;
+                }
+                else {
+                        g_free (connection->uri);
+                        g_free (connection);
+                        *result = indexed_search_result;
+                        return NULL;
+                }
                 return connection;
         }
         if (connection->method == MEDUSA_SEARCH_METHOD_UNINDEXED_ONLY) {
                 connection->indexed_search = NULL;
                 connection->unindexed_search = medusa_unindexed_search_new (result, uri);
-                connection->mode = SEARCH_MODE_UNINDEXED;
-                return connection;
+                if (result == GNOME_VFS_OK) {
+                        connection->mode = SEARCH_MODE_UNINDEXED;
+                        return connection;
+                }
+                else {
+                        g_free (connection->uri);
+                        g_free (connection);
+                        return NULL;
+                }
         }
+
         g_assert_not_reached ();
         return NULL;
 }
@@ -130,14 +200,15 @@ medusa_search_service_connection_start_search (MedusaSearchServiceConnection *co
         GnomeVFSResult result;
 
 	result = GNOME_VFS_OK;
-
-        if (connection->method != MEDUSA_SEARCH_METHOD_UNINDEXED_ONLY) {
-                result = medusa_indexed_search_start_search (connection->indexed_search,
-                                                             connection->uri);
+        if (connection->mode == SEARCH_MODE_INDEXED_AND_UNINDEXED ||
+            connection->mode == SEARCH_MODE_UNINDEXED) {
+                medusa_unindexed_search_start_search (connection->unindexed_search);
         }
 
-        if (connection->method != MEDUSA_SEARCH_METHOD_INDEX_ONLY) {
-                medusa_unindexed_search_start_search (connection->unindexed_search);
+        if (connection->mode == SEARCH_MODE_INDEXED ||
+            connection->mode == SEARCH_MODE_INDEXED_AND_UNINDEXED) {
+                result = medusa_indexed_search_start_search (connection->indexed_search,
+                                                             connection->uri);
         }
 
         return result;
@@ -146,28 +217,29 @@ medusa_search_service_connection_start_search (MedusaSearchServiceConnection *co
 
 GnomeVFSResult 
 medusa_search_service_connection_read_search_result (MedusaSearchServiceConnection *connection,
+                                                     GnomeVFSContext *context,
                                                      char **result)
 {
         GnomeVFSResult read_result;
 
-        if (connection->mode == SEARCH_MODE_INDEXED) {
+        if (connection->mode == SEARCH_MODE_INDEXED ||
+            (connection->mode == SEARCH_MODE_INDEXED_AND_UNINDEXED &&
+             !connection->finished_reading_indexed_results)) {
                 read_result = medusa_indexed_search_read_search_result (connection->indexed_search,
                                                                         result);
                 /* If we're doing a full search and we're out of 
                    results, switch gears into an unindexed search */
                 if (read_result == GNOME_VFS_ERROR_EOF &&
-                    connection->method == MEDUSA_SEARCH_METHOD_FULL_SEARCH) {
-                        connection->mode = SEARCH_MODE_UNINDEXED;
-                        return medusa_search_service_connection_read_search_result (connection, result);
-                } else {
-                        return read_result;
-                }
+                    connection->mode == SEARCH_MODE_INDEXED_AND_UNINDEXED) {
+                        connection->finished_reading_indexed_results = TRUE;
+                        return medusa_search_service_connection_read_search_result (connection, 
+                                                                                    context,
+                                                                                    result);
+                } 
         }
-        read_result = medusa_unindexed_search_read_search_result (connection->unindexed_search,
-                                                                  result);
-        if (read_result == GNOME_VFS_ERROR_DIRECTORY_BUSY) {
-                sleep (1);
+        else {
                 read_result = medusa_unindexed_search_read_search_result (connection->unindexed_search,
+                                                                          context,
                                                                           result);
         }
         return read_result;
@@ -178,13 +250,8 @@ medusa_search_service_connection_read_search_result (MedusaSearchServiceConnecti
 void medusa_search_service_connection_destroy (MedusaSearchServiceConnection *connection)
 {
         g_free (connection->uri);
-        if (connection->method != MEDUSA_SEARCH_METHOD_UNINDEXED_ONLY) {
-                medusa_indexed_search_destroy (connection->indexed_search);
-        }
-        if (connection->method != MEDUSA_SEARCH_METHOD_INDEX_ONLY) {
-                medusa_unindexed_search_destroy (connection->unindexed_search);
-        }
-
+        medusa_indexed_search_destroy (connection->indexed_search);
+        medusa_unindexed_search_destroy (connection->unindexed_search);
         g_free (connection);
 }
 
@@ -201,6 +268,7 @@ uri_has_valid_header (const char *uri)
         
         method_string = strchr (uri, ':') + 1;
         if (!(medusa_str_has_prefix (method_string, "index-only") ||
+              medusa_str_has_prefix (method_string, "index-if-available") ||
               medusa_str_has_prefix (method_string, "index-with-backup") ||
               medusa_str_has_prefix (method_string, "unindexed-only") ||
               medusa_str_has_prefix (method_string, "[file:///]"))) {
@@ -210,24 +278,27 @@ uri_has_valid_header (const char *uri)
         return TRUE;
 }
 
-static MedusaSearchMethod
-gnome_vfs_uri_to_search_method (GnomeVFSURI *gnome_vfs_uri)
+static gboolean
+vfs_uri_to_search_method (GnomeVFSURI *gnome_vfs_uri,
+                          MedusaSearchMethod *method)
 {
         char *uri;
         char *unescaped_uri;
-        MedusaSearchMethod method;
+        gboolean uri_looks_valid;
 
         uri = uri_to_string_remove_extra_slashes (gnome_vfs_uri);
         unescaped_uri = gnome_vfs_unescape_string (uri, NULL);
 
-        method = uri_to_search_method (unescaped_uri);
+        uri_looks_valid = uri_to_search_method (unescaped_uri,
+                                                method);
         g_free (unescaped_uri);
-        return method;
+        return uri_looks_valid;
 
 }
 
-static MedusaSearchMethod      
-uri_to_search_method (const char *uri)
+static gboolean
+uri_to_search_method (const char *uri,
+                      MedusaSearchMethod *search_method)
 {
         const char *method_string;
         
@@ -237,23 +308,32 @@ uri_to_search_method (const char *uri)
                   medusa_str_has_prefix (uri, "gnome-search:") ||
                   medusa_str_has_prefix (uri, "medusa:"));
         method_string = strchr (uri, ':') + 1;
-        
-        if (medusa_str_has_prefix (method_string, "index-only")) {
-                return MEDUSA_SEARCH_METHOD_INDEX_ONLY;
+
+        /* The uri is too short, return FALSE, as it cannot possibly be
+           valid */
+        if (*method_string == 0) {
+                return FALSE;
         }
-        if (medusa_str_has_prefix (method_string, "index-with-backup")) {
-                return MEDUSA_SEARCH_METHOD_FULL_SEARCH;
+        if (medusa_str_has_prefix (method_string, "index-only")) {
+                *search_method = MEDUSA_SEARCH_METHOD_INDEX_ONLY;
+        }
+        else if (medusa_str_has_prefix (method_string, "index-if-available")) {
+                *search_method = MEDUSA_SEARCH_METHOD_INDEX_IF_AVAILABLE;
+        }
+        else if (medusa_str_has_prefix (method_string, "index-with-backup")) {
+                *search_method = MEDUSA_SEARCH_METHOD_FULL_SEARCH;
         } 
-        if (medusa_str_has_prefix (method_string, "unindexed-only")) {
-                return MEDUSA_SEARCH_METHOD_UNINDEXED_ONLY;
+        else if (medusa_str_has_prefix (method_string, "unindexed-only")) {
+                *search_method = MEDUSA_SEARCH_METHOD_UNINDEXED_ONLY;
         }
         /* Check also for the case of no stated method, and return the default */
-        if (*method_string == '[') {
-                return MEDUSA_SEARCH_METHOD_INDEX_ONLY;
+        else if (*method_string == '[') {
+                *search_method = MEDUSA_SEARCH_METHOD_INDEX_ONLY;
         }
-        
-        g_assert_not_reached ();
-        return MEDUSA_SEARCH_METHOD_INDEX_ONLY;
+        else {
+                return FALSE;
+        }
+        return TRUE;
 }
 
 
