@@ -32,6 +32,7 @@
 #define MEDUSA_SEARCH_SERVICE_H
 
 #include <libgnomevfs/gnome-vfs-types.h>
+#include <libgnomevfs/gnome-vfs-private-types.h>
 
 #ifndef AF_LOCAL
 #define AF_LOCAL AF_UNIX
@@ -47,18 +48,26 @@
        + strlen ((ptr)->sun_path))
 #endif
 
-/* FIXME bugzilla.eazel.com 3008:  Is anyone using this type? */
-
-typedef struct MedusaSearchHandle MedusaSearchHandle;
-
 typedef struct MedusaSearchServiceConnection MedusaSearchServiceConnection;
 
 typedef enum {
-        MEDUSA_SEARCH_METHOD_INDEX_ONLY,
-        MEDUSA_SEARCH_METHOD_FULL_SEARCH,
-        MEDUSA_SEARCH_METHOD_UNINDEXED_ONLY
+        MEDUSA_SEARCH_METHOD_INDEX_ONLY, /* Search only the index.  Fail if it's not available. */
+        MEDUSA_SEARCH_METHOD_INDEX_IF_AVAILABLE, /* Search the index if it's there, otherwise do a slow search */
+        MEDUSA_SEARCH_METHOD_FULL_SEARCH,  /* Search the index, then do a slow search to confirm and add to results */
+        MEDUSA_SEARCH_METHOD_UNINDEXED_ONLY /* Don't use the index at all */
 } MedusaSearchMethod;
 
+typedef enum {
+        MEDUSA_SEARCH_OK,
+        MEDUSA_SEARCH_ERROR_INVALID_URI,
+        MEDUSA_SEARCH_ERROR_SERVICE_UNAVAILABLE,
+        MEDUSA_SEARCH_ERROR_SERVICE_BUSY,
+        MEDUSA_SEARCH_NO_RESULTS,
+        MEDUSA_SEARCH_ALL_TRUE,                    /* Return this if the search would return every result in the database */
+        MEDUSA_SEARCH_DATE_TOO_NEW,                /* Return this for an indexed search where the date requirements are
+                                                      too new to possibly match any result in the database */
+        MEDUSA_SEARCH_EOF
+} MedusaSearchResult;
 
 MedusaSearchServiceConnection *medusa_search_service_connection_new (const char *uri,
                                                                      GnomeVFSResult *result);
@@ -69,6 +78,7 @@ GnomeVFSResult medusa_search_service_connection_is_available_for_uri       (Gnom
 GnomeVFSResult medusa_search_service_connection_start_search       (MedusaSearchServiceConnection *connection);
 
 GnomeVFSResult medusa_search_service_connection_read_search_result (MedusaSearchServiceConnection *connection,
+                                                                    GnomeVFSContext *context,
                                                                     char **result);
 
 void           medusa_search_service_connection_destroy            (MedusaSearchServiceConnection *connection);
