@@ -34,6 +34,8 @@
 #include <glib.h>
 #include <libgnomevfs/gnome-vfs-file-info.h>
 #include <libgnomevfs/gnome-vfs-types.h>
+#include <libgnomevfs/gnome-vfs-uri.h>
+#include <libgnomevfs/gnome-vfs-context.h>
 #include <regex.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -43,11 +45,13 @@
 #include <unistd.h>
 
 #include "medusa-file-info-utilities.h"
+#include "medusa-string.h"
 #include "medusa-service-private.h"
 #include "medusa-unindexed-search.h"
 #include "medusa-unsearched-locations.h"
 #include "medusa-utils.h"
 
+
 typedef int MedusaEmblemData;
 
 #define UNINDEXED_RESULTS_FILE "/tmp/unindexed-results-"
@@ -157,12 +161,6 @@ static gboolean                criterion_field_is             (const char *crite
                                                                const char *field_name);   
 static int                     initialize_results_file_for_writing  (char *file_name);
 static int                     initialize_results_file_for_reading  (char *file_name);
-/* FIXME bugzilla.eazel.com 3003: 
-   Perhaps we should use the gnome vfs functions that 
-   do the same things */
-static char *                  uri_to_file_name               (const char *uri);
-static char *                  uri_to_path_name               (const char *uri);
-
 static char *                  match_string_to_date           (const char *match_string);
 
 static const char *            second_word_of                 (const char *criterion);
@@ -176,6 +174,10 @@ medusa_unindexed_search_new (GnomeVFSResult *result,
         GList *other_criteria;
         MedusaUnindexedSearch *search;
   
+        if (medusa_unindexed_search_is_available_for_uri (uri) != GNOME_VFS_OK) {
+                *result = GNOME_VFS_ERROR_SERVICE_NOT_AVAILABLE;
+                return NULL;
+        }
         search = g_new0 (MedusaUnindexedSearch, 1);
         search->root_directory = get_root_directory_from_uri (uri);
         search_string = uri_to_search_string (uri);
@@ -209,6 +211,7 @@ medusa_unindexed_search_new (GnomeVFSResult *result,
         /* For now, we can tell a search won't work
            if it contains criteria we don't know about */
         search->invalid_search = (g_list_length (other_criteria) > 0);
+        medusa_g_list_free_deep (other_criteria);
 
         /* Initialize other side of the search connection to
            write results to */
@@ -227,21 +230,37 @@ medusa_unindexed_search_new (GnomeVFSResult *result,
         
 }
 
+GnomeVFSResult
+medusa_unindexed_search_is_available_for_uri (const char *uri)
+{
+        g_return_val_if_fail (uri != NULL, GNOME_VFS_ERROR_INTERNAL);
+        if (strstr (uri, "& content ") == NULL &&
+            strstr (uri, "]content ") == NULL) {
+                return GNOME_VFS_OK;
+        }
+        return GNOME_VFS_ERROR_SERVICE_NOT_AVAILABLE;
+}
+
 GnomeVFSResult
 medusa_unindexed_search_start_search (MedusaUnindexedSearch *search)
 
 {
         pid_t result_process;
         
+        if (search->invalid_search) {
+                return GNOME_VFS_ERROR_INVALID_URI;
+        }
         result_process = fork ();
         if (result_process == 0) {
                 search_directory (search->root_directory, search);
+#ifdef MEDUSA_UNINDEXED_SEARCH_DEBUG
                 printf ("Search process is complete\n");
+#endif
                 write (search->output_fd, 
                        END_OF_UNINDEXED_SEARCH_RESULTS,
                        strlen (END_OF_UNINDEXED_SEARCH_RESULTS));
                 medusa_unindexed_search_destroy (search);
-                exit (1);
+                exit (0);
         }
         close (search->output_fd);
         return GNOME_VFS_OK;
@@ -249,6 +268,7 @@ medusa_unindexed_search_start_search (MedusaUnindexedSearch *search)
 
 GnomeVFSResult         
 medusa_unindexed_search_read_search_result (MedusaUnindexedSearch *search,
+                                            GnomeVFSContext *context,
                                             char **result)
 { 
         char tmpbuf[513];
@@ -257,7 +277,6 @@ medusa_unindexed_search_read_search_result (MedusaUnindexedSearch *search,
         int len;
         char *buffer;
 
-        
         g_return_val_if_fail (search != NULL, GNOME_VFS_ERROR_BAD_PARAMETERS);
 
         if (search->buffer == NULL) {
@@ -265,15 +284,23 @@ medusa_unindexed_search_read_search_result (MedusaUnindexedSearch *search,
         }
 
         while (strchr (search->buffer, '\n') == NULL) {
+                if (gnome_vfs_context_check_cancellation (context)) {
+                        return GNOME_VFS_ERROR_INTERRUPTED;
+                }
+                memset (&tmpbuf, 0, 513);
                 len = read (search->read_fd, &tmpbuf, 512 * sizeof (char)); 
-                if (len == -1) {
+                if (len == 0) {
+                        sleep (1);
+                }
+                else if (len == -1) {
                         *result = NULL;
-                        return GNOME_VFS_ERROR_DIRECTORY_BUSY;
+                        return GNOME_VFS_ERROR_EOF;
+                }
+                else {
+                        buffer = g_strconcat (search->buffer, tmpbuf, NULL);
+                        g_free (search->buffer);
+                        search->buffer = buffer;
                 }
-                tmpbuf[len] = '\0';
-                buffer = g_strconcat (search->buffer, tmpbuf, NULL);
-                g_free (search->buffer);
-                search->buffer = buffer;
         }
 
         
@@ -297,8 +324,16 @@ medusa_unindexed_search_read_search_result (MedusaUnindexedSearch *search,
 void                   
 medusa_unindexed_search_destroy (MedusaUnindexedSearch *search)
 {
+        if (search == NULL) {
+                return;
+        }
         medusa_unsearched_locations_shutdown ();
-        medusa_g_list_free_deep (search->criteria);
+        close (search->read_fd);
+        unlink (search->output_file_name);
+        medusa_g_list_free_deep (search->name_criteria);
+        medusa_g_list_free_deep (search->inode_criteria);
+        medusa_g_list_free_deep (search->mime_type_criteria);
+        medusa_g_list_free_deep (search->keywords_criteria);
         g_free (search->root_directory);
         g_free (search);
 }
@@ -356,7 +391,8 @@ search_directory (char *directory_name,
 		}
 		file_uri = medusa_full_uri_from_directory_and_file (directory_name,
                                                                     directory_entry->d_name);
-                if (file_matches_criteria (file_uri, 
+                if (!medusa_is_unsearched_location (file_uri) &&
+                    file_matches_criteria (file_uri, 
                                            search, 
                                            &file_is_directory,
                                            keywords_hash_table,
@@ -367,11 +403,11 @@ search_directory (char *directory_name,
                         
                         write_result = write (search->output_fd, file_uri, strlen (file_uri));
                         
-                        if (write_result == 0) {
+                        if (write_result <= 0) {
                                 /* FIXME bugzilla.eazel.com 3007: 
                                    We need to determine
                                    the correct error behavior in this case */
-                                g_warning ("The results aren't getting written to the socket\n");
+                                exit (1);
                         }
                         write_result = write (search->output_fd, "\n", strlen ("\n"));
                 }
@@ -558,40 +594,44 @@ matches_all_keywords_criteria (GList *keywords_criteria,
 
 static gboolean
 matches_one_name (const char *criterion,
-                const char *file_uri)
+                  const char *file_uri)
             
 {
-        char *name;
+        const char *name;
         const char *match_string;
         const char *match_relation;
         static GHashTable *regex_patterns = NULL;
         regex_t *pattern_data;
+        GnomeVFSURI *vfs_uri;
 
+        vfs_uri = gnome_vfs_uri_new (file_uri);
+        /* FIXME: This value is leaked in the get_dirname case */
         if (criterion_field_is (criterion, MEDUSA_FILE_NAME_CRITERION)) {
-                name = uri_to_file_name (file_uri);
+                name = gnome_vfs_uri_get_basename (vfs_uri);
         }
         else {
-                name = uri_to_path_name (file_uri);
+                name = gnome_vfs_uri_extract_dirname (vfs_uri);
         }
+        gnome_vfs_uri_unref (vfs_uri);
         match_relation = second_word_of (criterion);
         match_string = last_word_of (criterion);
         if (medusa_str_has_prefix (match_relation, "is")) {
-                return (strcmp (match_string, name) == 0);
+                return (strcasecmp (match_string, name) == 0);
         }
         if (medusa_str_has_prefix (match_relation, "is_not")) {
-                return (strcmp (match_string, name) != 0);
+                return (strcasecmp (match_string, name) != 0);
         }
         if (medusa_str_has_prefix (match_relation, "contains")) {
-                return (strstr (name, match_string) != NULL);
+                return medusa_strstr_case_insensitive (name, match_string);
         }
         if (medusa_str_has_prefix (match_relation, "does_not_contain")) {
-                return (strstr (name, match_string) == NULL);
+                return !medusa_strstr_case_insensitive (name, match_string);
         }
         if (medusa_str_has_prefix (match_relation, "begins_with")) {
-                return medusa_str_has_prefix (name, match_string);
+                return medusa_strcase_has_prefix (name, match_string);
         }
         if (medusa_str_has_prefix (match_relation, "ends_with")) {
-                return medusa_str_has_suffix (name, match_string);
+                return medusa_strcase_has_suffix (name, match_string);
         }
         if (medusa_str_has_prefix (match_relation, "matches")) {
                 return (fnmatch (match_string, name, 0));
@@ -981,39 +1021,6 @@ criterion_field_is (const char *criterion,
 }
 
 
-static char *                  
-uri_to_file_name (const char *uri)
-{
-        const char *location;
-
-        g_return_val_if_fail (uri != NULL, NULL);
-        g_return_val_if_fail (strchr (uri, '/') != NULL, NULL);
-        
-        location = uri;
-        location += strlen (uri);
-        while (*location != '/') {
-                location--;
-        }
-        return g_strdup (location);
-}
-
-static char *                  
-uri_to_path_name (const char *uri)
-{
-        const char *location;
-
-        g_return_val_if_fail (uri != NULL, NULL);
-        g_return_val_if_fail (strchr (uri, '/') != NULL, NULL);
-
-        location = uri;
-        location += strlen (uri);
-        while (*location != '/') {
-                location--;
-        }
-        return g_strndup (uri, location - uri);
-
-}
-
 static char *                  
 match_string_to_date (const char *match_string)
 {
