@@ -33,6 +33,7 @@
 #include <pwd.h>
 #include <stdio.h>
 #include <string.h>
+#include <sys/time.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include <xml-error.h>
@@ -91,51 +92,6 @@ str_ends_with_slash (const char *string)
 
 }
 
-gboolean
-medusa_group_contains (int gid, int uid)
-{
-        struct group *group;
-        struct passwd *password_entry;
-        int i;
-        group = getgrgid (gid);
-        if (group == NULL) {
-                return FALSE;
-        }
-        for (i = 0; group->gr_mem[i] != NULL; i++) {
-                password_entry = getpwnam (group->gr_mem[i]);
-                if (password_entry->pw_uid == uid) {
-                        return TRUE;
-                }
-        }
-        return FALSE;
-  
-}
-
-
-gboolean
-medusa_str_has_prefix (const char *string,
-                       const char *prefix)
-{
-        g_return_val_if_fail (string != NULL, FALSE);
-        g_return_val_if_fail (prefix != NULL, FALSE);
-
-        return strlen (prefix) <= strlen (string) &&
-                strncmp (string, prefix, strlen (prefix)) == 0;
-}
-
-
-gboolean
-medusa_str_has_suffix (const char *string,
-                       const char *suffix)
-{
-        g_return_val_if_fail (string != NULL, FALSE);
-        g_return_val_if_fail (suffix != NULL, FALSE);
-
-        return strlen (suffix) <= strlen (string) &&
-                strcmp (string + strlen (string) - strlen (suffix), 
-                        suffix) == 0;
-}
-
 
 /**
  * medusa_g_list_partition
@@ -200,6 +156,22 @@ medusa_g_list_partition (GList	          *list,
 }
 
 
+gboolean
+medusa_g_list_forall (GList *list,
+                      MedusaGPredicateFunc predicate,
+                      gpointer user_data)
+{
+        GList *p;
+
+        for (p = list; p != NULL; p = p->next) {
+                if (!predicate (p->data, user_data)) {
+                        return FALSE;
+                }
+        }
+        
+        return TRUE;
+}
+
 gint32 *             
 medusa_intersect_two_descending_integer_lists (gint32 *first_list,
                                                int first_list_entry_count,
@@ -655,3 +627,17 @@ medusa_keyword_hash_table_destroy (GHashTable *keywords_hash_table)
         /* It's just a str-to-str-list hash table. */
         medusa_str_to_str_list_hash_table_destroy (keywords_hash_table);
 }
+
+/**
+ * medusa_get_system_time
+ * 
+ * Return value: number of microseconds since the machine was turned on
+ */
+gint64
+medusa_get_system_time (void)
+{
+	struct timeval tmp;
+
+	gettimeofday (&tmp, NULL);
+	return (gint64)tmp.tv_usec + (gint64)tmp.tv_sec * G_GINT64_CONSTANT (1000000);
+}
