@@ -37,7 +37,7 @@
 #include <unistd.h>
 
 #include "medusa-search-service-private.h"
-#include <libmedusa/medusa-utils.h>
+#include <libmedusa/medusa-file-info-utilities.h>
 
 
 #define   CACHED_PERMISSION_NOT_CACHED 0
@@ -98,9 +98,16 @@ medusa_authenticate_create_cookie_file (char *request_line)
         g_return_if_fail (cookie_fd != -1);
         key = get_user_key (uid, pid);
         printf ("Making cookie with key %d\n", key);
-        g_return_if_fail (write (cookie_fd, &key, sizeof (int)) != -1);
-        g_return_if_fail (fchown (cookie_fd, uid, -1) != -1);
-        g_return_if_fail (close (cookie_fd) != -1);
+        
+        if (write (cookie_fd, &key, sizeof (int)) == -1) {
+                g_warning ("Authenticate write failed\n");
+        }
+        if (fchown (cookie_fd, uid, -1) == -1) {
+                g_warning ("Changing the owner of the cookie file failed\n");
+        }
+        if (close (cookie_fd) == -1) {
+                g_warning ("Couldn't close the cookie file\n");
+        }
   
 }
 
