@@ -40,8 +40,8 @@
 
 
 
-static void                 run_query                        (MedusaMasterDB *master_db,
-                                                              char *search_uri,
+static void                 handle_authenticated_request     (MedusaMasterDB *master_db,
+                                                              const char *search_uri,
                                                               int uid, 
                                                               int client_fd);
 static char *               get_uid_pid_key_information      (char *line,
@@ -63,11 +63,13 @@ static void                 free_entries                       (gpointer key,
 static gboolean             another_transmission_line_exists   (char *line);
 static void                 send_cookie_creation_acknowledgement (int client_fd);
 
+
+
 void
 medusa_file_search_parse_transmission (char *transmission, int client_fd,
 				       MedusaMasterDB *db)
 {
-        char *line;
+        char *line, *error_string;
         int uid, pid, key;
 
         g_return_if_fail (strlen (transmission) != 0);
@@ -80,41 +82,106 @@ medusa_file_search_parse_transmission (char *transmission, int client_fd,
                 if (medusa_authenticate_is_cookie_request (line)) {
                         medusa_authenticate_create_cookie_file (line); 
                         send_cookie_creation_acknowledgement (client_fd);
-                }
-                else {
+                } else  {
                         line = get_uid_pid_key_information (line, &uid, &pid, &key);
                         if (line == NULL) {
-                                g_warning ("Search daemon is trying to parse NULL transmission to its input socket\n");
+                                medusa_log_event ("Invalid search request transmission\n",
+                                                  MEDUSA_DB_LOG_ERRORS);
                                 return;
                         }
                         if (medusa_authenticate_is_correct_cookie (uid, pid, key)) {
 #ifdef SEARCH_DAEMON_DEBUG
                                 printf ("Key %d matched for uid %d and pid %d\n", key, uid, pid);
 #endif
-                                run_query (db, line, uid, client_fd);
+                                handle_authenticated_request (db, line, uid, client_fd);
+                        }
+                        else {
+                                error_string = g_strdup_printf ("Invalid authentication received from user claiming to have uid %d", uid);
+                                medusa_log_event (error_string, MEDUSA_DB_LOG_ERRORS);
+                                g_free (error_string);
                         }
                 }
                 line = goto_next_line (line);
         }
 }  
 
+static gboolean
+transmit_string (const char *string,
+                 int fd)
+{
+        int write_result;
+
+        g_assert (string != NULL);
+        write_result = write (fd, string, strlen (string));
+        if (write_result != strlen (string)) {
+                return FALSE;
+        }
+        write_result = write (fd, "\n", 1);
+        if (write_result != 1) {
+                return FALSE;
+        }
+        
+        return TRUE;
+                
+}
+
+static gboolean
+send_back_parsing_results (MedusaParsedSearchURI *parsed_search_uri,
+                           int client_fd)
+{
+        switch (parsed_search_uri->error) {
+
+        case MEDUSA_SEARCH_URI_NO_ERROR:
+                transmit_string (SEARCH_URI_OK_TRANSMISSION, client_fd);
+                return TRUE;
+        case MEDUSA_SEARCH_URI_IS_ALWAYS_TRUE:
+                transmit_string (SEARCH_URI_ALL_RESULTS_VALID_TRANSMISSION, client_fd);
+                return FALSE;
+        case MEDUSA_SEARCH_URI_IS_ALWAYS_FALSE:
+                transmit_string (SEARCH_URI_NO_RESULTS_TRANSMISSION, client_fd);
+                return FALSE;
+        case MEDUSA_SEARCH_URI_OBSOLETES_INDEX:
+                transmit_string (SEARCH_URI_INDEX_OBSOLETE_ERROR_TRANSMISSION, client_fd);
+                return FALSE;
+        case MEDUSA_SEARCH_URI_SYNTAX_ERROR:
+                transmit_string (SEARCH_URI_SYNTAX_ERROR_TRANSMISSION, client_fd);
+                return FALSE;
+        default:
+                g_assert_not_reached ();
+                return FALSE;
+        }
+}
+                           
 
 static void
-run_query (MedusaMasterDB *db, 
-           char *search_uri,
-           int uid,
-           int client_fd) 
+handle_authenticated_request (MedusaMasterDB *db, 
+                              const char *search_uri,
+                              int uid,
+                              int client_fd) 
 {
+        MedusaParsedSearchURI *parsed_search_uri;
         GList *results;
         char *search_uri_to_send;
         
         printf ("Running query\n");
+        /* Do initial parsing of uri, and send back errors that occur during 
+           this process */
         /* Do preprocessing on the search uri, to add extra permission 
            or file type checks, if relevant */
+        /* FIXME bugzilla.eazel.com 4887: This should be a query "optimization" and not here */
         search_uri_to_send = medusa_search_uri_add_extra_needed_criteria (search_uri,
                                                                           uid);
-        results = medusa_master_db_query (db, search_uri_to_send);
-        g_free (search_uri_to_send);
+
+        parsed_search_uri = medusa_search_uri_parse (search_uri_to_send, db, uid);
+        if (!send_back_parsing_results (parsed_search_uri,
+                                        client_fd)) {
+                g_free (search_uri_to_send);
+                medusa_parsed_search_uri_free (parsed_search_uri);
+                return;
+        }
+        g_free (search_uri_to_send);        
+        results = medusa_master_db_run_search (db, parsed_search_uri);
+        medusa_parsed_search_uri_free (parsed_search_uri);
         printf ("Done running query\n");
         present_query_results (results, client_fd, uid);
 
@@ -130,8 +197,7 @@ get_uid_pid_key_information (char *line, int *uid, int *pid, int *key)
         result = strchr (line,'\t');
         if (result == NULL) {
                 return result;
-        }
-        else {
+        } else {
                 /* return the next meaningful character, rather than the tab */
                 return result+1;
         }
@@ -152,14 +218,12 @@ present_query_results (GList *results,
                         memset (send_buffer, 0, MAX_LINE);
                         strncpy (send_buffer, iterator->data, MAX_LINE);
                         send_buffer [MAX_LINE] = 0;
-                        write (client_fd, send_buffer, strlen (send_buffer));
-                        write (client_fd, "\n", 1);
+                        transmit_string (send_buffer, client_fd);
                 }
         }
         g_hash_table_foreach (read_cache, free_entries, NULL);
         g_hash_table_destroy (read_cache);
-        write (client_fd, SEARCH_END_TRANSMISSION, strlen (SEARCH_END_TRANSMISSION));
-        write (client_fd, "\n", 1);
+        transmit_string (SEARCH_END_TRANSMISSION, client_fd);
         medusa_g_list_free_deep (results);
 
         printf ("Finished sending search results\n");
@@ -195,6 +259,6 @@ static void
 send_cookie_creation_acknowledgement (int client_fd)
 {
         g_return_if_fail (client_fd != -1);
-
+        g_print ("writing new cookie\n");
         write (client_fd, COOKIE_REQUEST, strlen (COOKIE_REQUEST));
 }
